@@ -33,8 +33,8 @@
 #include "catch.hpp"
 #include "helpers.h"
 #include "tiledb/sm/cpp_api/tiledb"
+#include "tiledb/sm/filesystem/uri.h"
 #include "tiledb/sm/misc/constants.h"
-#include "tiledb/sm/misc/uri.h"
 #include "tiledb/sm/misc/utils.h"
 
 using namespace tiledb;
@@ -112,6 +112,7 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic]") {
     CHECK(schema.domain().dimensions()[1].tile_extent<int>() == 5);
     CHECK(schema.domain().cell_num() == 20301);
   }
+
   SECTION("Make Buffer") {
     Array array(ctx, "cpp_unit_array", TILEDB_WRITE);
     Query query(ctx, array, TILEDB_WRITE);
@@ -191,7 +192,6 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic]") {
       query.set_offsets_buffer("a4", a4buf.first);
       query.set_data_buffer("a5", a5);
       query.set_layout(TILEDB_ROW_MAJOR);
-
       REQUIRE(query.submit() == Query::Status::COMPLETE);
 
       // check a1 buffers
@@ -210,7 +210,6 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic]") {
       REQUIRE(buf_back_elem_size == sizeof(char));
       REQUIRE(offsets_back == a2buf.first.data());
       REQUIRE(offsets_back_nelem == 2);
-
       CHECK(!query.has_results());
 
       query.finalize();
@@ -348,8 +347,17 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic]") {
     query.set_layout(TILEDB_GLOBAL_ORDER);
     CHECK(query.submit() == tiledb::Query::Status::COMPLETE);
     REQUIRE_NOTHROW(query.finalize());
+
+    // Check non-empty domain while array open in write mode
+    CHECK_THROWS(array.non_empty_domain<int>(1));
+    CHECK_THROWS(array.non_empty_domain<int>("d1"));
+
     array.close();
 
+    // Check non-empty domain before open from read
+    CHECK_THROWS(array.non_empty_domain<int>(1));
+    CHECK_THROWS(array.non_empty_domain<int>("d1"));
+
     array.open(TILEDB_READ);
 
     // Check non-empty domain
@@ -1282,6 +1290,7 @@ TEST_CASE(
   CHECK_THROWS(array_r.non_empty_domain_var("foo"));
 
   // Read
+  bool empty_results = false;
   std::string s1("a", 1);
   std::string s2("ee", 2);
   Query query_r(ctx, array_r, TILEDB_READ);
@@ -1317,6 +1326,7 @@ TEST_CASE(
     std::array<std::string, 2> range = query_r.range(0, 0);
     CHECK(range[0] == s1);
     CHECK(range[1] == "");
+    empty_results = true;
   }
 
   SECTION("Empty ranges") {
@@ -1328,6 +1338,7 @@ TEST_CASE(
     std::array<std::string, 2> range = query_r.range(0, 0);
     CHECK(range[0] == "");
     CHECK(range[1] == "");
+    empty_results = true;
   }
 
   std::string data;
@@ -1336,8 +1347,17 @@ TEST_CASE(
   query_r.set_data_buffer("d", data);
   query_r.set_offsets_buffer("d", offsets);
   query_r.submit();
-  CHECK(data == "aabbccdddd");
-  std::vector<uint64_t> c_offsets = {0, 2, 4, 6};
+  if (empty_results) {
+    auto data_str = data.c_str();
+    for (uint64_t i = 0; i < 10; i++) {
+      CHECK(data_str[i] == 0);
+    }
+  } else {
+    CHECK(data == "aabbccdddd");
+  }
+  std::vector<uint64_t> c_offsets(4, 0);
+  if (!empty_results)
+    c_offsets = {0, 2, 4, 6};
   CHECK(offsets == c_offsets);
   auto ret = query.result_buffer_elements();
   REQUIRE(ret.size() == 2);
