@@ -33,6 +33,7 @@
 #include "catch.hpp"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/cpp_api/tiledb_experimental"
+#include "tiledb/sm/misc/constants.h"
 
 #include <limits>
 
@@ -116,6 +117,7 @@ TEST_CASE("C++ API: Schema", "[cppapi][schema]") {
     CHECK(schema.attribute("a3").cell_val_num() == 2);
     CHECK(schema.attribute("a4").cell_val_num() == TILEDB_VAR_NUM);
     CHECK(schema.attribute("a4").type() == TILEDB_UINT32);
+    CHECK(schema.version() == tiledb::sm::constants::format_version);
 
     auto dims = schema.domain().dimensions();
     REQUIRE(dims.size() == 2);
@@ -177,6 +179,7 @@ TEST_CASE("C++ API: Schema", "[cppapi][schema]") {
     CHECK(schema.attribute("a4").cell_val_num() == TILEDB_VAR_NUM);
     CHECK(schema.attribute("a4").type() == TILEDB_UINT32);
     CHECK(schema.allows_dups() == true);
+    CHECK(schema.version() == tiledb::sm::constants::format_version);
 
     auto dims = schema.domain().dimensions();
     REQUIRE(dims.size() == 2);
@@ -402,6 +405,10 @@ TEST_CASE(
   // drop attribute a1
   evolution.drop_attribute("a1");
 
+  uint64_t now = tiledb_timestamp_now_ms();
+  now = now + 1;
+  evolution.set_timestamp_range({now, now});
+
   // evolve array
   evolution.array_evolve(array_uri);
 
@@ -505,7 +512,9 @@ TEST_CASE(
 
   // Evolve
   {
+    uint64_t now = tiledb_timestamp_now_ms() + 1;
     ArraySchemaEvolution schemaEvolution = ArraySchemaEvolution(ctx);
+    schemaEvolution.set_timestamp_range(std::make_pair(now, now));
 
     // Add attribute b
     Attribute b = Attribute::create<uint32_t>(ctx, "b");
@@ -651,4 +660,4 @@ TEST_CASE(
   if (vfs.is_dir(array_uri)) {
     vfs.remove_dir(array_uri);
   }
-}
+}
\ No newline at end of file
