@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -37,7 +37,11 @@
 #include <unordered_map>
 #include <vector>
 
+#include "tiledb/common/common.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/common/status.h"
+#include "tiledb/sm/array/array_directory.h"
+#include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/crypto/encryption_key.h"
 #include "tiledb/sm/fragment/fragment_info.h"
 #include "tiledb/sm/metadata/metadata.h"
@@ -76,11 +80,22 @@ class Array {
   /*                API                */
   /* ********************************* */
 
+  /** Returns the array directory object. */
+  const ArrayDirectory& array_directory() const;
+
+  /** Sets the latest array schema.
+   * @param array_schema The array schema to set.
+   */
+  void set_array_schema_latest(const shared_ptr<ArraySchema>& array_schema);
+
   /** Returns the latest array schema. */
-  ArraySchema* array_schema_latest() const;
+  const ArraySchema& array_schema_latest() const;
+
+  /** Returns the latest array schema as a shared pointer. */
+  shared_ptr<const ArraySchema> array_schema_latest_ptr() const;
 
   /** Returns array schemas map. */
-  inline const std::unordered_map<std::string, tdb_shared_ptr<ArraySchema>>&
+  inline const std::unordered_map<std::string, shared_ptr<ArraySchema>>&
   array_schemas_all() const {
     return array_schemas_all_;
   }
@@ -166,7 +181,7 @@ class Array {
    * Returns the fragment metadata of the array. If the array is not open,
    * an empty vector is returned.
    */
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragment_metadata() const;
+  std::vector<shared_ptr<FragmentMetadata>> fragment_metadata() const;
 
   /**
    * Returns `true` if the array is empty at the time it is opened.
@@ -181,7 +196,7 @@ class Array {
   bool is_remote() const;
 
   /** Retrieves the array schema. Errors if the array is not open. */
-  Status get_array_schema(ArraySchema** array_schema) const;
+  tuple<Status, optional<shared_ptr<ArraySchema>>> get_array_schema() const;
 
   /** Retrieves the query type. Errors if the array is not open. */
   Status get_query_type(QueryType* qyery_type) const;
@@ -247,9 +262,6 @@ class Array {
   /** Retrieves a reference to the array config. */
   Config config() const;
 
-  /** Directly set the array URI. */
-  Status set_uri(const std::string& uri);
-
   /** Directly set the array URI for serialized compatibility with pre
    * TileDB 2.5 clients */
   Status set_uri_serialized(const std::string& uri);
@@ -339,34 +351,39 @@ class Array {
    * fetch the underlying Metadata object to set the values we are loading from
    * REST. A lock should already by taken before load_metadata is called.
    */
-  Metadata* metadata();
+  Metadata* unsafe_metadata();
 
   /** Returns the non-empty domain of the opened array.
    *  If the non_empty_domain has not been computed or loaded
    *  it will be loaded first
    * */
-  const NDRange& non_empty_domain();
+  tuple<Status, optional<const NDRange>> non_empty_domain();
 
   /** Returns the non-empty domain of the opened array. */
   void set_non_empty_domain(const NDRange& non_empty_domain);
 
+  /** Returns the memory tracker. */
+  MemoryTracker* memory_tracker();
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
   /** The latest array schema. */
-  ArraySchema* array_schema_latest_;
+  shared_ptr<ArraySchema> array_schema_latest_;
 
   /**
    * A map of all array_schemas_all
    */
-  std::unordered_map<std::string, tdb_shared_ptr<ArraySchema>>
-      array_schemas_all_;
+  std::unordered_map<std::string, shared_ptr<ArraySchema>> array_schemas_all_;
 
   /** The array URI. */
   URI array_uri_;
 
+  /** The array directory object for listing URIs. */
+  ArrayDirectory array_dir_;
+
   /** This is a backwards compatible URI from serialization
    *  In TileDB 2.5 we removed sending the URI but 2.4 and older were
    * unconditionally setting the URI, so things got set to an empty stirng Now
@@ -382,10 +399,10 @@ class Array {
    * bytes should be stored. Whenever a key is needed, a pointer to this
    * memory region should be passed instead of a copy of the bytes.
    */
-  tdb_shared_ptr<EncryptionKey> encryption_key_;
+  shared_ptr<EncryptionKey> encryption_key_;
 
   /** The metadata of the fragments the array was opened with. */
-  std::vector<tdb_shared_ptr<FragmentMetadata>> fragment_metadata_;
+  std::vector<shared_ptr<FragmentMetadata>> fragment_metadata_;
 
   /** `True` if the array has been opened. */
   std::atomic<bool> is_open_;
@@ -433,9 +450,6 @@ class Array {
    */
   std::vector<uint8_t> last_max_buffer_sizes_subarray_;
 
-  /** Mutex for thread-safety. */
-  mutable std::mutex mtx_;
-
   /** True if the array is remote (has `tiledb://` URI scheme). */
   bool remote_;
 
@@ -451,6 +465,9 @@ class Array {
   /** The non-empty domain of the array. */
   NDRange non_empty_domain_;
 
+  /** Memory tracker for the array. */
+  MemoryTracker memory_tracker_;
+
   /* ********************************* */
   /*          PRIVATE METHODS          */
   /* ********************************* */
