@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  * @copyright Copyright (c) 2016 MIT and Intel Corporation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -32,6 +32,7 @@
  */
 
 #include "tiledb/sm/array_schema/array_schema.h"
+#include "tiledb/common/common.h"
 #include "tiledb/common/heap_memory.h"
 #include "tiledb/common/logger.h"
 #include "tiledb/sm/array_schema/attribute.h"
@@ -45,6 +46,8 @@
 #include "tiledb/sm/enums/layout.h"
 #include "tiledb/sm/filter/compression_filter.h"
 #include "tiledb/sm/misc/hilbert.h"
+#include "tiledb/sm/misc/tdb_time.h"
+#include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <cassert>
 #include <iostream>
@@ -87,33 +90,98 @@ ArraySchema::ArraySchema(ArrayType array_type)
   cell_validity_filters_.add_filter(CompressionFilter(
       constants::cell_validity_compression,
       constants::cell_validity_compression_level));
+
+  // Generate URI and name for ArraySchema
+  generate_uri();
 }
 
-ArraySchema::ArraySchema(const ArraySchema* array_schema) {
-  allows_dups_ = array_schema->allows_dups_;
-  array_uri_ = array_schema->array_uri_;
-  uri_ = array_schema->uri_;
-  array_type_ = array_schema->array_type_;
+ArraySchema::ArraySchema(
+    URI uri,
+    uint32_t version,
+    ArrayType array_type,
+    bool allows_dups,
+    shared_ptr<Domain> domain,
+    Layout cell_order,
+    Layout tile_order,
+    uint64_t capacity,
+    std::vector<shared_ptr<const Attribute>> attributes,
+    FilterPipeline cell_var_offsets_filters,
+    FilterPipeline cell_validity_filters,
+    FilterPipeline coords_filters)
+    : uri_(uri)
+    , version_(version)
+    , array_type_(array_type)
+    , allows_dups_(allows_dups)
+    , domain_(domain)
+    , cell_order_(cell_order)
+    , tile_order_(tile_order)
+    , capacity_(capacity)
+    , attributes_(attributes)
+    , cell_var_offsets_filters_(cell_var_offsets_filters)
+    , cell_validity_filters_(cell_validity_filters)
+    , coords_filters_(coords_filters) {
+  Status st;
+  // Populate timestamp range
+  st = utils::parse::get_timestamp_range(uri_, &timestamp_range_);
+  if (!st.ok())
+    throw StatusException(st);
+
+  // Set schema name
+  name_ = uri_.last_path_part();
+
+  // Create dimension map
+  dim_map_.clear();
+  for (unsigned d = 0; d < domain_->dim_num(); ++d) {
+    auto dim = domain_->dimension(d);
+    dim_map_[dim->name()] = dim;
+  }
+
+  // Create attribute map
+  if (!attributes_.empty()) {
+    auto attr = attributes_.back().get();
+    attribute_map_[attr->name()] = attr;
+  }
+
+  st = check_double_delta_compressor(coords_filters_);
+  if (!st.ok())
+    throw StatusException(
+        Status_ArraySchemaError("Array schema check failed; Double delta "
+                                "compression used in zipped coords."));
+
+  st = check_string_compressor(coords_filters_);
+  if (!st.ok())
+    throw StatusException(Status_ArraySchemaError(
+        "Array schema check failed; RLE compression used."));
+
+  if (!check_attribute_dimension_names())
+    throw StatusException(
+        Status_ArraySchemaError("Array schema check failed; Attributes "
+                                "and dimensions must have unique names"));
+}
+
+ArraySchema::ArraySchema(const ArraySchema& array_schema) {
+  allows_dups_ = array_schema.allows_dups_;
+  array_uri_ = array_schema.array_uri_;
+  uri_ = array_schema.uri_;
+  array_type_ = array_schema.array_type_;
   domain_ = nullptr;
-  timestamp_range_ = array_schema->timestamp_range_;
+  timestamp_range_ = array_schema.timestamp_range_;
 
-  capacity_ = array_schema->capacity_;
-  cell_order_ = array_schema->cell_order_;
-  cell_var_offsets_filters_ = array_schema->cell_var_offsets_filters_;
-  cell_validity_filters_ = array_schema->cell_validity_filters_;
-  coords_filters_ = array_schema->coords_filters_;
-  tile_order_ = array_schema->tile_order_;
-  version_ = array_schema->version_;
+  capacity_ = array_schema.capacity_;
+  cell_order_ = array_schema.cell_order_;
+  cell_var_offsets_filters_ = array_schema.cell_var_offsets_filters_;
+  cell_validity_filters_ = array_schema.cell_validity_filters_;
+  coords_filters_ = array_schema.coords_filters_;
+  tile_order_ = array_schema.tile_order_;
+  version_ = array_schema.version_;
 
-  set_domain(array_schema->domain_);
+  set_domain(array_schema.domain_);
 
   attribute_map_.clear();
-  for (auto attr : array_schema->attributes_)
+  for (auto attr : array_schema.attributes_)
     add_attribute(attr, false);
 
-  // This has to be the last thing set because add_attribute sets the name
-  // TODO: This behavior needs to be changed
-  name_ = array_schema->name_;
+  name_ = array_schema.name_;
 }
 
 ArraySchema::~ArraySchema() {
@@ -138,7 +206,7 @@ const URI& ArraySchema::array_uri() const {
 
 const Attribute* ArraySchema::attribute(unsigned int id) const {
   if (id < attributes_.size())
-    return attributes_[id];
+    return attributes_[id].get();
   return nullptr;
 }
 
@@ -151,7 +219,8 @@ unsigned int ArraySchema::attribute_num() const {
   return (unsigned)attributes_.size();
 }
 
-const std::vector<Attribute*>& ArraySchema::attributes() const {
+const std::vector<shared_ptr<const Attribute>>& ArraySchema::attributes()
+    const {
   return attributes_;
 }
 
@@ -219,15 +288,15 @@ const FilterPipeline& ArraySchema::cell_validity_filters() const {
 Status ArraySchema::check() const {
   if (domain_ == nullptr)
     return LOG_STATUS(
-        Status::ArraySchemaError("Array schema check failed; Domain not set"));
+        Status_ArraySchemaError("Array schema check failed; Domain not set"));
 
   auto dim_num = this->dim_num();
   if (dim_num == 0)
-    return LOG_STATUS(Status::ArraySchemaError(
+    return LOG_STATUS(Status_ArraySchemaError(
         "Array schema check failed; No dimensions provided"));
 
   if (cell_order_ == Layout::HILBERT && dim_num > Hilbert::HC_MAX_DIM) {
-    return LOG_STATUS(Status::ArraySchemaError(
+    return LOG_STATUS(Status_ArraySchemaError(
         "Array schema check failed; Maximum dimensions supported by Hilbert "
         "order exceeded"));
   }
@@ -236,21 +305,22 @@ Status ArraySchema::check() const {
     auto type = domain_->dimension(0)->type();
     if (datatype_is_real(type)) {
       return LOG_STATUS(
-          Status::ArraySchemaError("Array schema check failed; Dense arrays "
-                                   "cannot have floating point domains"));
+          Status_ArraySchemaError("Array schema check failed; Dense arrays "
+                                  "cannot have floating point domains"));
     }
     if (attributes_.size() == 0) {
-      return LOG_STATUS(Status::ArraySchemaError(
+      return LOG_STATUS(Status_ArraySchemaError(
           "Array schema check failed; No attributes provided"));
     }
   }
 
-  RETURN_NOT_OK(check_double_delta_compressor());
+  RETURN_NOT_OK(check_double_delta_compressor(coords_filters()));
+  RETURN_NOT_OK(check_string_compressor(coords_filters()));
 
   if (!check_attribute_dimension_names())
     return LOG_STATUS(
-        Status::ArraySchemaError("Array schema check failed; Attributes "
-                                 "and dimensions must have unique names"));
+        Status_ArraySchemaError("Array schema check failed; Attributes "
+                                "and dimensions must have unique names"));
 
   // Success
   return Status::Ok();
@@ -262,7 +332,7 @@ Status ArraySchema::check_attributes(
     if (attr == constants::coords)
       continue;
     if (attribute_map_.find(attr) == attribute_map_.end())
-      return LOG_STATUS(Status::ArraySchemaError(
+      return LOG_STATUS(Status_ArraySchemaError(
           "Attribute check failed; cannot find attribute"));
   }
 
@@ -293,11 +363,12 @@ bool ArraySchema::dense() const {
   return array_type_ == ArrayType::DENSE;
 }
 
-const Dimension* ArraySchema::dimension(unsigned int i) const {
+shared_ptr<const Dimension> ArraySchema::dimension(unsigned int i) const {
   return domain_->dimension(i);
 }
 
-const Dimension* ArraySchema::dimension(const std::string& name) const {
+shared_ptr<const Dimension> ArraySchema::dimension(
+    const std::string& name) const {
   auto it = dim_map_.find(name);
   return it == dim_map_.end() ? nullptr : it->second;
 }
@@ -386,7 +457,7 @@ bool ArraySchema::is_field(const std::string& name) const {
 }
 
 bool ArraySchema::is_nullable(const std::string& name) const {
-  const Attribute* const attr = this->attribute(name);
+  auto attr = this->attribute(name);
   if (attr == nullptr)
     return false;
   return attr->nullable();
@@ -491,25 +562,24 @@ bool ArraySchema::var_size(const std::string& name) const {
   return false;
 }
 
-Status ArraySchema::add_attribute(const Attribute* attr, bool check_special) {
+Status ArraySchema::add_attribute(
+    shared_ptr<const Attribute> attr, bool check_special) {
   // Sanity check
   if (attr == nullptr)
-    return LOG_STATUS(Status::ArraySchemaError(
+    return LOG_STATUS(Status_ArraySchemaError(
         "Cannot add attribute; Input attribute is null"));
 
   // Do not allow attributes with special names
   if (check_special && attr->name().find(constants::special_name_prefix) == 0) {
     std::string msg = "Cannot add attribute; Attribute names starting with '";
     msg += std::string(constants::special_name_prefix) + "' are reserved";
-    return LOG_STATUS(Status::ArraySchemaError(msg));
+    return LOG_STATUS(Status_ArraySchemaError(msg));
   }
 
   // Create new attribute and potentially set a default name
-  auto new_attr = tdb_new(Attribute, attr);
-  attributes_.emplace_back(new_attr);
-  attribute_map_[new_attr->name()] = new_attr;
+  attributes_.emplace_back(attr);
+  attribute_map_[attr->name()] = attr.get();
 
-  RETURN_NOT_OK(generate_uri());
   return Status::Ok();
 }
 
@@ -517,96 +587,193 @@ Status ArraySchema::drop_attribute(const std::string& attr_name) {
   std::lock_guard<std::mutex> lock(mtx_);
   if (attr_name.empty()) {
     return LOG_STATUS(
-        Status::ArraySchemaError("Cannot remove an empty name attribute"));
+        Status_ArraySchemaError("Cannot remove an empty name attribute"));
   }
 
   if (attribute_map_.find(attr_name) == attribute_map_.end()) {
     // Not exists.
     return LOG_STATUS(
-        Status::ArraySchemaError("Cannot remove a non-exist attribute"));
+        Status_ArraySchemaError("Cannot remove a non-exist attribute"));
   }
   attribute_map_.erase(attr_name);
 
   // Iterate backwards and remove the attribute pointer, it should be slightly
   // faster than iterating forward.
-  std::vector<Attribute*>::iterator it;
+  decltype(attributes_)::iterator it;
   for (it = attributes_.end(); it != attributes_.begin();) {
     --it;
     if ((*it)->name() == attr_name) {
-      tdb_delete(*it);
       it = attributes_.erase(it);
     }
   }
-  RETURN_NOT_OK(generate_uri());
+
   return Status::Ok();
 }
 
-Status ArraySchema::deserialize(ConstBuffer* buff) {
+// #TODO Add security validation on incoming URI
+ArraySchema ArraySchema::deserialize(ConstBuffer* buff, const URI& uri) {
+  Status st;
   // Load version
-  RETURN_NOT_OK(buff->read(&version_, sizeof(uint32_t)));
+  // #TODO Add security validation
+  uint32_t version;
+  st = buff->read(&version, sizeof(uint32_t));
+  if (!st.ok())
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Failed to load version.");
+  if (!(version <= constants::format_version))
+    throw StatusException(Status_ArraySchemaError(
+        "[ArraySchema::deserialize] Incompatible format version."));
 
   // Load allows_dups
-  if (version_ >= 5)
-    RETURN_NOT_OK(buff->read(&allows_dups_, sizeof(bool)));
+  // Note: No security validation is possible.
+  bool allows_dups = false;
+  if (version >= 5) {
+    st = buff->read(&allows_dups, sizeof(bool));
+    if (!st.ok())
+      throw std::runtime_error(
+          "[ArraySchema::deserialize] Failed to load allows_dups.");
+  }
 
   // Load array type
-  uint8_t array_type;
-  RETURN_NOT_OK(buff->read(&array_type, sizeof(uint8_t)));
-  array_type_ = (ArrayType)array_type;
+  uint8_t array_type_loaded;
+  st = buff->read(&array_type_loaded, sizeof(uint8_t));
+  if (!st.ok())
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Failed to load array type.");
+  if (!array_type_is_valid(array_type_loaded).ok())
+    throw StatusException(Status_ArraySchemaError(
+        "[ArraySchema::deserialize] Invalid array type."));
+  ArrayType array_type = ArrayType(array_type_loaded);
 
   // Load tile order
-  uint8_t tile_order;
-  RETURN_NOT_OK(buff->read(&tile_order, sizeof(uint8_t)));
-  tile_order_ = (Layout)tile_order;
+  uint8_t tile_order_loaded;
+  st = buff->read(&tile_order_loaded, sizeof(uint8_t));
+  if (!st.ok())
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Failed to load tile order.");
+  if (!tile_order_is_valid(tile_order_loaded).ok())
+    throw StatusException(Status_ArraySchemaError(
+        "[ArraySchema::deserialize] Invalid tile order."));
+  Layout tile_order = Layout(tile_order_loaded);
 
   // Load cell order
-  uint8_t cell_order;
-  RETURN_NOT_OK(buff->read(&cell_order, sizeof(uint8_t)));
-  cell_order_ = (Layout)cell_order;
+  uint8_t cell_order_loaded;
+  st = buff->read(&cell_order_loaded, sizeof(uint8_t));
+  if (!st.ok())
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Failed to load cell order.");
+  if (!cell_order_is_valid(cell_order_loaded).ok())
+    throw StatusException(Status_ArraySchemaError(
+        "[ArraySchema::deserialize] Invalid cell order."));
+  Layout cell_order = Layout(cell_order_loaded);
 
   // Load capacity
-  RETURN_NOT_OK(buff->read(&capacity_, sizeof(uint64_t)));
+  // #TODO Add security validation
+  uint64_t capacity;
+  st = buff->read(&capacity, sizeof(uint64_t));
+  if (!st.ok())
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Failed to load capacity.");
 
   // Load coords filters
-  RETURN_NOT_OK(coords_filters_.deserialize(buff));
+  // Note: Security validation delegated to invoked API
+  // #TODO Add security validation
+  auto&& [st_coords_filters, coords_filters]{
+      FilterPipeline::deserialize(buff, version)};
+  if (!st_coords_filters.ok()) {
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Cannot deserialize coords filters.");
+  }
 
   // Load offsets filters
-  RETURN_NOT_OK(cell_var_offsets_filters_.deserialize(buff));
+  // Note: Security validation delegated to invoked API
+  // #TODO Add security validation
+  auto&& [st_cell_var_filters, cell_var_filters]{
+      FilterPipeline::deserialize(buff, version)};
+  if (!st_coords_filters.ok()) {
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Cannot deserialize cell var filters.");
+  }
 
   // Load validity filters
-  if (version_ >= 7)
-    RETURN_NOT_OK(cell_validity_filters_.deserialize(buff));
+  // Note: Security validation delegated to invoked API
+  // #TODO Add security validation
+  FilterPipeline cell_validity_filters;
+  if (version >= 7) {
+    auto&& [st_cell_validity_filters, cell_validity_filters_deserialized]{
+        FilterPipeline::deserialize(buff, version)};
+    if (!st_cell_validity_filters.ok()) {
+      throw std::runtime_error(
+          "[ArraySchema::deserialize] Cannot deserialize cell validity "
+          "filters.");
+    }
+    cell_validity_filters = cell_validity_filters_deserialized.value();
+  }
 
   // Load domain
-  domain_ = tdb_new(Domain);
-  RETURN_NOT_OK(domain_->deserialize(buff, version_));
+  // Note: Security validation delegated to invoked API
+  // #TODO Add security validation
+  auto&& [st_domain, domain]{
+      Domain::deserialize(buff, version, cell_order, tile_order)};
+  if (!st_domain.ok()) {
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Cannot deserialize domain.");
+  }
 
   // Load attributes
+  // Note: Security validation delegated to invoked API
+  // #TODO Add security validation
+  std::vector<shared_ptr<const Attribute>> attributes;
   uint32_t attribute_num;
-  RETURN_NOT_OK(buff->read(&attribute_num, sizeof(uint32_t)));
+  st = buff->read(&attribute_num, sizeof(uint32_t));
+  if (!st.ok())
+    throw std::runtime_error(
+        "[ArraySchema::deserialize] Cannot load attribute_num.");
   for (uint32_t i = 0; i < attribute_num; ++i) {
-    auto attr = tdb_new(Attribute);
-    RETURN_NOT_OK_ELSE(attr->deserialize(buff, version_), tdb_delete(attr));
-    attributes_.emplace_back(attr);
-    attribute_map_[attr->name()] = attr;
+    auto&& [st_attr, attr]{Attribute::deserialize(buff, version)};
+    if (!st_attr.ok()) {
+      throw std::runtime_error(
+          "[ArraySchema::deserialize] Cannot deserialize attributes.");
+    }
+
+    attributes.emplace_back(make_shared<Attribute>(HERE(), move(attr.value())));
   }
 
-  // Create dimension map
-  auto dim_num = domain()->dim_num();
-  for (unsigned d = 0; d < dim_num; ++d) {
-    auto dim = dimension(d);
-    dim_map_[dim->name()] = dim;
+  // Validate
+  if (cell_order == Layout::HILBERT &&
+      domain.value()->dim_num() > Hilbert::HC_MAX_DIM) {
+    throw StatusException(Status_ArraySchemaError(
+        "Array schema check failed; Maximum dimensions supported by Hilbert "
+        "order exceeded"));
   }
 
-  // Initialize the rest of the object members
-  RETURN_NOT_OK(init());
+  if (array_type == ArrayType::DENSE) {
+    auto type = domain.value()->dimension(0)->type();
+    if (datatype_is_real(type)) {
+      throw StatusException(
+          Status_ArraySchemaError("Array schema check failed; Dense arrays "
+                                  "cannot have floating point domains"));
+    }
+    if (attributes.size() == 0) {
+      throw StatusException(Status_ArraySchemaError(
+          "Array schema check failed; No attributes provided"));
+    }
+  }
 
   // Success
-  return Status::Ok();
-}
-
-const Domain* ArraySchema::domain() const {
-  return domain_;
+  return ArraySchema(
+      uri,
+      version,
+      array_type,
+      allows_dups,
+      domain.value(),
+      cell_order,
+      tile_order,
+      capacity,
+      attributes,
+      cell_var_filters.value(),
+      cell_validity_filters,
+      coords_filters.value());
 }
 
 Status ArraySchema::init() {
@@ -622,7 +789,7 @@ Status ArraySchema::init() {
 
 Status ArraySchema::set_allows_dups(bool allows_dups) {
   if (allows_dups && array_type_ == ArrayType::DENSE)
-    return LOG_STATUS(Status::ArraySchemaError(
+    return LOG_STATUS(Status_ArraySchemaError(
         "Dense arrays cannot allow coordinate duplicates"));
 
   allows_dups_ = allows_dups;
@@ -642,6 +809,10 @@ void ArraySchema::set_capacity(uint64_t capacity) {
 }
 
 Status ArraySchema::set_coords_filter_pipeline(const FilterPipeline* pipeline) {
+  assert(pipeline);
+  RETURN_NOT_OK(check_string_compressor(*pipeline));
+  RETURN_NOT_OK(check_double_delta_compressor(*pipeline));
+
   coords_filters_ = *pipeline;
   return Status::Ok();
 }
@@ -655,8 +826,8 @@ Status ArraySchema::set_cell_var_offsets_filter_pipeline(
 Status ArraySchema::set_cell_order(Layout cell_order) {
   if (dense() && cell_order == Layout::HILBERT)
     return LOG_STATUS(
-        Status::ArraySchemaError("Cannot set cell order; Hilbert order is only "
-                                 "applicable to sparse arrays"));
+        Status_ArraySchemaError("Cannot set cell order; Hilbert order is only "
+                                "applicable to sparse arrays"));
 
   cell_order_ = cell_order;
 
@@ -669,25 +840,25 @@ Status ArraySchema::set_cell_validity_filter_pipeline(
   return Status::Ok();
 }
 
-Status ArraySchema::set_domain(Domain* domain) {
+Status ArraySchema::set_domain(shared_ptr<Domain> domain) {
   if (domain == nullptr)
     return LOG_STATUS(
-        Status::ArraySchemaError("Cannot set domain; Input domain is nullptr"));
+        Status_ArraySchemaError("Cannot set domain; Input domain is nullptr"));
 
   if (domain->dim_num() == 0)
-    return LOG_STATUS(Status::ArraySchemaError(
+    return LOG_STATUS(Status_ArraySchemaError(
         "Cannot set domain; Domain must contain at least one dimension"));
 
   if (array_type_ == ArrayType::DENSE) {
     if (!domain->all_dims_same_type())
       return LOG_STATUS(
-          Status::ArraySchemaError("Cannot set domain; In dense arrays, all "
-                                   "dimensions must have the same datatype"));
+          Status_ArraySchemaError("Cannot set domain; In dense arrays, all "
+                                  "dimensions must have the same datatype"));
 
     auto type = domain->dimension(0)->type();
     if (!datatype_is_integer(type) && !datatype_is_datetime(type) &&
         !datatype_is_time(type)) {
-      return LOG_STATUS(Status::ArraySchemaError(
+      return LOG_STATUS(Status_ArraySchemaError(
           std::string("Cannot set domain; Dense arrays "
                       "do not support dimension datatype '") +
           datatype_str(type) + "'"));
@@ -699,8 +870,7 @@ Status ArraySchema::set_domain(Domain* domain) {
   }
 
   // Set domain
-  tdb_delete(domain_);
-  domain_ = tdb_new(Domain, domain);
+  domain_ = domain;
 
   // Create dimension map
   dim_map_.clear();
@@ -715,7 +885,7 @@ Status ArraySchema::set_domain(Domain* domain) {
 
 Status ArraySchema::set_tile_order(Layout tile_order) {
   if (tile_order == Layout::HILBERT)
-    return LOG_STATUS(Status::ArraySchemaError(
+    return LOG_STATUS(Status_ArraySchemaError(
         "Cannot set tile order; Hilbert order is not applicable to tiles"));
 
   tile_order_ = tile_order;
@@ -751,48 +921,14 @@ uint64_t ArraySchema::timestamp_start() const {
   return timestamp_range_.first;
 }
 
-URI ArraySchema::uri() {
-  std::lock_guard<std::mutex> lock(mtx_);
-  if (uri_.is_invalid()) {
-    generate_uri();
-  }
-  URI result = uri_;
-  return result;
+const URI& ArraySchema::uri() const {
+  return uri_;
 }
 
-void ArraySchema::set_uri(const URI& uri) {
-  std::lock_guard<std::mutex> lock(mtx_);
-  uri_ = uri;
-  name_ = uri_.last_path_part();
-  utils::parse::get_timestamp_range(uri_, &timestamp_range_);
-}
-
-Status ArraySchema::get_uri(URI* uri) {
-  if (uri_.is_invalid()) {
-    return LOG_STATUS(
-        Status::ArraySchemaError("Error in ArraySchema; invalid URI"));
-  }
-  *uri = uri_;
-  return Status::Ok();
-}
-
-std::string ArraySchema::name() {
-  std::lock_guard<std::mutex> lock(mtx_);
-  if (name_.empty()) {
-    generate_uri();
-  }
+const std::string& ArraySchema::name() const {
   return name_;
 }
 
-Status ArraySchema::get_name(std::string* name) const {
-  if (name_.empty()) {
-    return LOG_STATUS(
-        Status::ArraySchemaError("Error in ArraySchema; Empty name"));
-  }
-  *name = name_;
-  return Status::Ok();
-}
-
 /* ****************************** */
 /*         PRIVATE METHODS        */
 /* ****************************** */
@@ -807,11 +943,12 @@ bool ArraySchema::check_attribute_dimension_names() const {
   return (names.size() == attributes_.size() + dim_num);
 }
 
-Status ArraySchema::check_double_delta_compressor() const {
+Status ArraySchema::check_double_delta_compressor(
+    const FilterPipeline& coords_filters) const {
   // Check if coordinate filters have DOUBLE DELTA as a compressor
   bool has_double_delta = false;
-  for (unsigned i = 0; i < coords_filters_.size(); ++i) {
-    if (coords_filters_.get_filter(i)->type() ==
+  for (unsigned i = 0; i < coords_filters.size(); ++i) {
+    if (coords_filters.get_filter(i)->type() ==
         FilterType::FILTER_DOUBLE_DELTA) {
       has_double_delta = true;
       break;
@@ -831,8 +968,42 @@ Status ArraySchema::check_double_delta_compressor() const {
     auto dim_type = dim->type();
     if (datatype_is_real(dim_type) && dim_filters.empty())
       return LOG_STATUS(
-          Status::ArraySchemaError("Real dimension cannot inherit coordinate "
-                                   "filters with DOUBLE DELTA compression"));
+          Status_ArraySchemaError("Real dimension cannot inherit coordinate "
+                                  "filters with DOUBLE DELTA compression"));
+  }
+
+  return Status::Ok();
+}
+
+Status ArraySchema::check_string_compressor(
+    const FilterPipeline& filters) const {
+  // There is no error if only 1 filter is used for RLE or Dictionary-encoding
+  if (filters.size() <= 1 ||
+      !(filters.has_filter(FilterType::FILTER_RLE) ||
+        filters.has_filter(FilterType::FILTER_DICTIONARY))) {
+    return Status::Ok();
+  }
+
+  // Error if there are also other filters set for a string dimension together
+  // with RLE or Dictionary-encoding
+  auto dim_num = domain_->dim_num();
+  for (unsigned d = 0; d < dim_num; ++d) {
+    auto dim = domain_->dimension(d);
+    const auto& dim_filters = dim->filters();
+    // if it's a var-length string dimension and there is no specific filter
+    // list already set for that dimension (then coords_filters_ will be used)
+    if (dim->type() == Datatype::STRING_ASCII && dim->var_size() &&
+        dim_filters.empty()) {
+      if (filters.has_filter(FilterType::FILTER_RLE)) {
+        return LOG_STATUS(Status_ArraySchemaError(
+            "RLE filter cannot be combined with other filters when applied to "
+            "variable length string dimensions"));
+      } else {
+        return LOG_STATUS(Status_ArraySchemaError(
+            "Dictionary-encoding filter cannot be combined with other filters "
+            "when applied to variable length string dimensions"));
+      }
+    }
   }
 
   return Status::Ok();
@@ -846,12 +1017,8 @@ void ArraySchema::clear() {
   capacity_ = constants::capacity;
   cell_order_ = Layout::ROW_MAJOR;
   tile_order_ = Layout::ROW_MAJOR;
-
-  for (auto& attr : attributes_)
-    tdb_delete(attr);
   attributes_.clear();
 
-  tdb_delete(domain_);
   domain_ = nullptr;
   timestamp_range_ = std::make_pair(0, 0);
 }
@@ -866,8 +1033,24 @@ Status ArraySchema::generate_uri() {
   ss << "__" << timestamp_range_.first << "_" << timestamp_range_.second << "_"
      << uuid;
   name_ = ss.str();
-  uri_ = array_uri_.join_path(constants::array_schema_folder_name)
-             .join_path(name_);
+  uri_ =
+      array_uri_.join_path(constants::array_schema_dir_name).join_path(name_);
+
+  return Status::Ok();
+}
+
+Status ArraySchema::generate_uri(
+    const std::pair<uint64_t, uint64_t>& timestamp_range) {
+  std::string uuid;
+  RETURN_NOT_OK(uuid::generate_uuid(&uuid, false));
+
+  timestamp_range_ = timestamp_range;
+  std::stringstream ss;
+  ss << "__" << timestamp_range_.first << "_" << timestamp_range_.second << "_"
+     << uuid;
+  name_ = ss.str();
+  uri_ =
+      array_uri_.join_path(constants::array_schema_dir_name).join_path(name_);
 
   return Status::Ok();
 }
