@@ -47,11 +47,11 @@ Status GZip::compress(
     int level, ConstBuffer* input_buffer, Buffer* output_buffer) {
   // Sanity check
   if (input_buffer->data() == nullptr || output_buffer->data() == nullptr)
-    return LOG_STATUS(Status::CompressionError(
+    return LOG_STATUS(Status_CompressionError(
         "Failed compressing with GZip; invalid buffer format"));
 
   if (level > GZip::maximum_level())
-    return LOG_STATUS(Status::CompressionError(
+    return LOG_STATUS(Status_CompressionError(
         "Failed compressing with GZip; invalid compression level."));
 
   int ret;
@@ -61,11 +61,12 @@ Status GZip::compress(
   strm.zalloc = Z_NULL;
   strm.zfree = Z_NULL;
   strm.opaque = Z_NULL;
-  ret = deflateInit(&strm, level < 0 ? GZip::default_level() : level);
+  ret =
+      deflateInit(&strm, level < level_limit_ ? GZip::default_level() : level);
 
   if (ret != Z_OK) {
     (void)deflateEnd(&strm);
-    return LOG_STATUS(Status::GZipError("Cannot compress with GZIP"));
+    return LOG_STATUS(Status_GZipError("Cannot compress with GZIP"));
   }
 
   // Compress
@@ -80,7 +81,7 @@ Status GZip::compress(
 
   // Return
   if (ret == Z_STREAM_ERROR || strm.avail_in != 0)
-    return LOG_STATUS(Status::GZipError("Cannot compress with GZIP"));
+    return LOG_STATUS(Status_GZipError("Cannot compress with GZIP"));
 
   // Set size of compressed data
   uint64_t compressed_size = output_buffer->free_space() - strm.avail_out;
@@ -90,11 +91,15 @@ Status GZip::compress(
   return Status::Ok();
 }
 
+Status GZip::compress(ConstBuffer* input_buffer, Buffer* output_buffer) {
+  return GZip::compress(GZip::default_level(), input_buffer, output_buffer);
+}
+
 Status GZip::decompress(
     ConstBuffer* input_buffer, PreallocatedBuffer* output_buffer) {
   // Sanity check
   if (input_buffer->data() == nullptr || output_buffer->data() == nullptr)
-    return LOG_STATUS(Status::CompressionError(
+    return LOG_STATUS(Status_CompressionError(
         "Failed decompressing with GZip; invalid buffer format"));
 
   int ret;
@@ -109,7 +114,7 @@ Status GZip::decompress(
   ret = inflateInit(&strm);
 
   if (ret != Z_OK) {
-    return LOG_STATUS(Status::GZipError("Cannot decompress with GZIP"));
+    return LOG_STATUS(Status_GZipError("Cannot decompress with GZIP"));
   }
 
   // Decompress
@@ -121,7 +126,7 @@ Status GZip::decompress(
 
   if (ret != Z_STREAM_END) {
     return LOG_STATUS(
-        Status::GZipError("Cannot decompress with GZIP, Stream Error"));
+        Status_GZipError("Cannot decompress with GZIP, Stream Error"));
   }
 
   // Set size of decompressed data
