@@ -37,6 +37,7 @@
 
 #include "tiledb.h"
 
+#include <cstddef>
 #include <cstdint>
 #include <cstring>
 #include <iterator>
@@ -80,10 +81,17 @@ template <typename T>
 struct type_to_tiledb {
   static_assert(IS_TRIVIALLY_COPYABLE(T), "Type must be trivially copyable.");
   using type = char;
-  static const tiledb_datatype_t tiledb_type = TILEDB_CHAR;
+  static const tiledb_datatype_t tiledb_type = TILEDB_STRING_ASCII;
   static constexpr const char* name = "Trivially Copyable (CHAR)";
 };
 
+template <>
+struct type_to_tiledb<std::byte> {
+  using type = std::byte;
+  static const tiledb_datatype_t tiledb_type = TILEDB_BLOB;
+  static constexpr const char* name = "BLOB";
+};
+
 template <>
 struct type_to_tiledb<int8_t> {
   using type = int8_t;
@@ -165,6 +173,13 @@ struct tiledb_to_type<TILEDB_CHAR> {
   static constexpr const char* name = "CHAR";
 };
 
+template <>
+struct tiledb_to_type<TILEDB_BLOB> {
+  using type = std::byte;
+  static const tiledb_datatype_t tiledb_type = TILEDB_BLOB;
+  static constexpr const char* name = "BLOB";
+};
+
 template <>
 struct tiledb_to_type<TILEDB_INT8> {
   using type = int8_t;
@@ -244,6 +259,7 @@ inline std::string type_to_str(tiledb_datatype_t type) {
 
 inline bool tiledb_string_type(tiledb_datatype_t type) {
   switch (type) {
+    case TILEDB_CHAR:
     case TILEDB_STRING_ASCII:
     case TILEDB_STRING_UTF8:
     case TILEDB_STRING_UTF16:
