@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -49,6 +49,11 @@ PositiveDeltaFilter::PositiveDeltaFilter()
   max_window_size_ = 1024;
 }
 
+PositiveDeltaFilter::PositiveDeltaFilter(uint32_t max_window_size)
+    : Filter(FilterType::FILTER_POSITIVE_DELTA)
+    , max_window_size_(max_window_size) {
+}
+
 void PositiveDeltaFilter::dump(FILE* out) const {
   if (out == nullptr)
     out = stdout;
@@ -56,11 +61,13 @@ void PositiveDeltaFilter::dump(FILE* out) const {
 }
 
 Status PositiveDeltaFilter::run_forward(
+    const Tile& tile,
+    Tile* const offsets_tile,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
     FilterBuffer* output) const {
-  auto tile_type = pipeline_->current_tile()->type();
+  auto tile_type = tile.type();
 
   // If encoding can't work, just return the input unmodified.
   if (!datatype_is_integer(tile_type)) {
@@ -69,30 +76,35 @@ Status PositiveDeltaFilter::run_forward(
     return Status::Ok();
   }
 
+  /* Note: Arithmetic operations cannot be performed on std::byte.
+    We will use uint8_t for the Datatype::BLOB case as it is the same size as
+    std::byte and can have arithmetic perfomed on it. */
   switch (tile_type) {
     case Datatype::INT8:
       return run_forward<int8_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
+    case Datatype::BLOB:
     case Datatype::UINT8:
       return run_forward<uint8_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT16:
       return run_forward<int16_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT16:
       return run_forward<uint16_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT32:
-      return run_forward<int>(input_metadata, input, output_metadata, output);
+      return run_forward<int>(
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT32:
       return run_forward<unsigned>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT64:
       return run_forward<int64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT64:
       return run_forward<uint64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::DATETIME_YEAR:
     case Datatype::DATETIME_MONTH:
     case Datatype::DATETIME_WEEK:
@@ -116,15 +128,17 @@ Status PositiveDeltaFilter::run_forward(
     case Datatype::TIME_FS:
     case Datatype::TIME_AS:
       return run_forward<int64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     default:
       return LOG_STATUS(
-          Status::FilterError("Cannot filter; Unsupported input type"));
+          Status_FilterError("Cannot filter; Unsupported input type"));
   }
 }
 
 template <typename T>
 Status PositiveDeltaFilter::run_forward(
+    const Tile&,
+    Tile* const,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
@@ -207,7 +221,7 @@ Status PositiveDeltaFilter::encode_part(
       for (uint32_t j = 0; j < window_nelts; j++) {
         T curr_value = input->value<T>();
         if (curr_value < prev_value)
-          return LOG_STATUS(Status::FilterError(
+          return LOG_STATUS(Status_FilterError(
               "Positive delta filter error: delta is not positive."));
 
         T delta = curr_value - prev_value;
@@ -223,6 +237,8 @@ Status PositiveDeltaFilter::encode_part(
 }
 
 Status PositiveDeltaFilter::run_reverse(
+    const Tile& tile,
+    Tile* const offsets_tile,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
@@ -230,7 +246,7 @@ Status PositiveDeltaFilter::run_reverse(
     const Config& config) const {
   (void)config;
 
-  auto tile_type = pipeline_->current_tile()->type();
+  auto tile_type = tile.type();
 
   // If encoding wasn't applied, just return the input unmodified.
   if (!datatype_is_integer(tile_type)) {
@@ -239,30 +255,35 @@ Status PositiveDeltaFilter::run_reverse(
     return Status::Ok();
   }
 
+  /* Note: Arithmetic operations cannot be performed on std::byte.
+    We will use uint8_t for the Datatype::BLOB case as it is the same size as
+    std::byte and can have arithmetic perfomed on it. */
   switch (tile_type) {
     case Datatype::INT8:
       return run_reverse<int8_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
+    case Datatype::BLOB:
     case Datatype::UINT8:
       return run_reverse<uint8_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT16:
       return run_reverse<int16_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT16:
       return run_reverse<uint16_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT32:
-      return run_reverse<int>(input_metadata, input, output_metadata, output);
+      return run_reverse<int>(
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT32:
       return run_reverse<unsigned>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::INT64:
       return run_reverse<int64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::UINT64:
       return run_reverse<uint64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     case Datatype::DATETIME_YEAR:
     case Datatype::DATETIME_MONTH:
     case Datatype::DATETIME_WEEK:
@@ -286,20 +307,22 @@ Status PositiveDeltaFilter::run_reverse(
     case Datatype::TIME_FS:
     case Datatype::TIME_AS:
       return run_reverse<int64_t>(
-          input_metadata, input, output_metadata, output);
+          tile, offsets_tile, input_metadata, input, output_metadata, output);
     default:
       return LOG_STATUS(
-          Status::FilterError("Cannot filter; Unsupported input type"));
+          Status_FilterError("Cannot filter; Unsupported input type"));
   }
 }
 
 template <typename T>
 Status PositiveDeltaFilter::run_reverse(
+    const Tile& tile,
+    Tile* const,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
     FilterBuffer* output) const {
-  auto tile_type = pipeline_->current_tile()->type();
+  auto tile_type = tile.type();
   auto tile_type_size = datatype_size(tile_type);
 
   uint32_t num_windows;
@@ -347,7 +370,7 @@ Status PositiveDeltaFilter::run_reverse(
 Status PositiveDeltaFilter::set_option_impl(
     FilterOption option, const void* value) {
   if (value == nullptr)
-    return LOG_STATUS(Status::FilterError(
+    return LOG_STATUS(Status_FilterError(
         "Positive delta filter error; invalid option value"));
 
   switch (option) {
@@ -356,7 +379,7 @@ Status PositiveDeltaFilter::set_option_impl(
       return Status::Ok();
     default:
       return LOG_STATUS(
-          Status::FilterError("Positive delta filter error; unknown option"));
+          Status_FilterError("Positive delta filter error; unknown option"));
   }
 }
 
@@ -368,7 +391,7 @@ Status PositiveDeltaFilter::get_option_impl(
       return Status::Ok();
     default:
       return LOG_STATUS(
-          Status::FilterError("Positive delta filter error; unknown option"));
+          Status_FilterError("Positive delta filter error; unknown option"));
   }
 }
 
@@ -386,11 +409,6 @@ PositiveDeltaFilter* PositiveDeltaFilter::clone_impl() const {
   return clone;
 }
 
-Status PositiveDeltaFilter::deserialize_impl(ConstBuffer* buff) {
-  RETURN_NOT_OK(buff->read(&max_window_size_, sizeof(uint32_t)));
-  return Status::Ok();
-}
-
 Status PositiveDeltaFilter::serialize_impl(Buffer* buff) const {
   RETURN_NOT_OK(buff->write(&max_window_size_, sizeof(uint32_t)));
   return Status::Ok();
