@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -59,17 +59,23 @@ extern const double est_result_size_amplification;
 /** Default fanout for RTrees. */
 extern const unsigned rtree_fanout;
 
-/** The object filelock name. */
-extern const std::string filelock_name;
-
 /** The array schema file name. */
 extern const std::string array_schema_filename;
 
-/** The array schema folder name. */
-extern const std::string array_schema_folder_name;
+/** The array schema directory name. */
+extern const std::string array_schema_dir_name;
 
-/** The array metadata folder name. */
-extern const std::string array_metadata_folder_name;
+/** The array metadata directory name. */
+extern const std::string array_metadata_dir_name;
+
+/** The array fragment metadata directory name. */
+extern const std::string array_fragment_meta_dir_name;
+
+/** The array fragments directory name. */
+extern const std::string array_fragments_dir_name;
+
+/** The array commit directory name. */
+extern const std::string array_commits_dir_name;
 
 /** The default tile capacity. */
 extern const uint64_t capacity;
@@ -176,9 +182,18 @@ extern const std::string vacuum_file_suffix;
 /** Suffix for the special ok files used in TileDB. */
 extern const std::string ok_file_suffix;
 
+/** Suffix for the special write files used in TileDB. */
+extern const std::string write_file_suffix;
+
 /** Suffix for the special metadata files used in TileDB. */
 extern const std::string meta_file_suffix;
 
+/** Suffix for the special consolidated commits files used in TileDB. */
+extern const std::string con_commits_file_suffix;
+
+/** Suffix for the special ignore files used in TileDB. */
+extern const std::string ignore_file_suffix;
+
 /** The fragment metadata file name. */
 extern const std::string fragment_metadata_filename;
 
@@ -197,6 +212,12 @@ extern uint64_t generic_tile_cell_size;
 /** The group file name. */
 extern const std::string group_filename;
 
+/** The group details directory name. */
+extern const std::string group_detail_dir_name;
+
+/** The group metadata directory name. */
+extern const std::string group_metadata_dir_name;
+
 /** The maximum number of bytes written in a single I/O. */
 extern const uint64_t max_write_bytes;
 
@@ -314,6 +335,9 @@ extern const std::string filter_checksum_md5_str;
 /** String describing FILTER_CHECKSUM_SHA256. */
 extern const std::string filter_checksum_sha256_str;
 
+/** String describing FILTER_DICTIONARY. */
+extern const std::string filter_dictionary_str;
+
 /** The string representation for FilterOption type compression_level. */
 extern const std::string filter_option_compression_level_str;
 
@@ -339,6 +363,9 @@ extern const std::string float64_str;
 /** The string representation for type char. */
 extern const std::string char_str;
 
+/** The string representation for type blob. */
+extern const std::string blob_str;
+
 /** The string representation for type int8. */
 extern const std::string int8_str;
 
@@ -572,6 +599,40 @@ extern const std::array<std::string, 6> cert_files_linux;
 
 /** Delimiter for lists passed as config parameter */
 extern const std::string config_delimiter;
+
+/** String describing MIME_AUTODETECT. */
+extern const std::string mime_autodetect_str;
+
+/** String describing MIME_TIFF. */
+extern const std::string mime_tiff_str;
+
+/** String describing MIME_PDF. */
+extern const std::string mime_pdf_str;
+
+/** The default tile extent used in filestore arrays. */
+extern const uint64_t filestore_default_tile_extent;
+
+/** Name of the single dimension used in filestore arrays. */
+extern const std::string filestore_dimension_name;
+
+/** Name of the single attribute used in filestore arrays. */
+extern const std::string filestore_attribute_name;
+
+/** Name of the metadata key used in filestore arrays for current size. */
+extern const std::string filestore_metadata_size_key;
+
+/** Name of the metadata key used in filestore arrays for mime type. */
+extern const std::string filestore_metadata_mime_type_key;
+
+/** Name of the metadata key used in filestore arrays for mime encoding. */
+extern const std::string filestore_metadata_mime_encoding_key;
+
+/** Name of the metadata key used in filestore arrays for original filename. */
+extern const std::string filestore_metadata_original_filename_key;
+
+/** Name of the metadata key used in filestore arrays for filename extension. */
+extern const std::string filestore_metadata_file_extension_key;
+
 }  // namespace constants
 
 }  // namespace sm
