@@ -65,9 +65,14 @@ class QueryCondition {
         : field_name_(std::move(field_name))
         , op_(op) {
       condition_value_data_.resize(condition_value_size);
-      condition_value_ =
-          condition_value == nullptr ? nullptr : condition_value_data_.data();
+      condition_value_ = nullptr;
+
       if (condition_value != nullptr) {
+        // Using an empty string litteral for empty string as vector::resize(0)
+        // doesn't guarantee an allocation.
+        condition_value_ = condition_value_size == 0 ?
+                               (void*)"" :
+                               condition_value_data_.data();
         memcpy(
             condition_value_data_.data(),
             condition_value,
@@ -184,7 +189,7 @@ class QueryCondition {
    * @param array_schema The current array schena.
    * @return Status
    */
-  Status check(const ArraySchema* array_schema) const;
+  Status check(const ArraySchema& array_schema) const;
 
   /**
    * Combines this instance with the right-hand-side instance by
@@ -221,8 +226,8 @@ class QueryCondition {
    * @return Status
    */
   Status apply(
-      const ArraySchema* array_schema,
-      std::vector<ResultCellSlab>* result_cell_slabs,
+      const ArraySchema& array_schema,
+      std::vector<ResultCellSlab>& result_cell_slabs,
       uint64_t stride) const;
 
   /**
@@ -237,9 +242,8 @@ class QueryCondition {
    * @param result_buffer The buffer to use for results.
    * @return Status
    */
-  template <typename T>
   Status apply_dense(
-      const ArraySchema* const array_schema,
+      const ArraySchema& array_schema,
       ResultTile* result_tile,
       const uint64_t start,
       const uint64_t length,
@@ -256,9 +260,9 @@ class QueryCondition {
    * @param cell_count The cell count after condition is applied.
    * @return Status
    */
-  template <typename T, typename BitmapType>
+  template <typename BitmapType>
   Status apply_sparse(
-      const ArraySchema* const array_schema,
+      const ArraySchema& array_schema,
       ResultTile& result_tile,
       std::vector<BitmapType>& result_bitmap,
       uint64_t* cell_count);
@@ -343,17 +347,16 @@ class QueryCondition {
    * @param nullable The attribute is nullable or not.
    * @param fill_value The fill value for the cells.
    * @param result_cell_slabs The input cell slabs.
-   * @param out_result_cell_slabs The filtered cell slabs.
+   * @return The filtered cell slabs.
    */
   template <typename T, QueryConditionOp Op>
-  void apply_clause(
+  std::vector<ResultCellSlab> apply_clause(
       const Clause& clause,
       uint64_t stride,
       const bool var_size,
       const bool nullable,
       const ByteVecValue& fill_value,
-      const std::vector<ResultCellSlab>& result_cell_slabs,
-      std::vector<ResultCellSlab>* out_result_cell_slabs) const;
+      const std::vector<ResultCellSlab>& result_cell_slabs) const;
 
   /**
    * Applies a clause on primitive-typed result cell slabs.
@@ -364,17 +367,16 @@ class QueryCondition {
    * @param nullable The attribute is nullable or not.
    * @param fill_value The fill value for the cells.
    * @param result_cell_slabs The input cell slabs.
-   * @param out_result_cell_slabs The filtered cell slabs.
+   * @return Status, filtered cell slabs.
    */
   template <typename T>
-  Status apply_clause(
+  tuple<Status, optional<std::vector<ResultCellSlab>>> apply_clause(
       const Clause& clause,
       uint64_t stride,
       const bool var_size,
       const bool nullable,
       const ByteVecValue& fill_value,
-      const std::vector<ResultCellSlab>& result_cell_slabs,
-      std::vector<ResultCellSlab>* out_result_cell_slabs) const;
+      const std::vector<ResultCellSlab>& result_cell_slabs) const;
 
   /**
    * Applies a clause to filter result cells from the input
@@ -384,14 +386,13 @@ class QueryCondition {
    * @param array_schema The current array schema.
    * @param stride The stride between cells.
    * @param result_cell_slabs The input cell slabs.
-   * @param out_result_cell_slabs The filtered cell slabs.
+   * @return Status, filtered cell slabs.
    */
-  Status apply_clause(
+  tuple<Status, optional<std::vector<ResultCellSlab>>> apply_clause(
       const QueryCondition::Clause& clause,
-      const ArraySchema* const array_schema,
+      const ArraySchema& array_schema,
       uint64_t stride,
-      const std::vector<ResultCellSlab>& result_cell_slabs,
-      std::vector<ResultCellSlab>* out_result_cell_slabs) const;
+      const std::vector<ResultCellSlab>& result_cell_slabs) const;
 
   /**
    * Applies a clause on a dense result tile,
@@ -404,9 +405,6 @@ class QueryCondition {
    * @param src_cell The cell offset in the source tile.
    * @param stride The stride between cells.
    * @param var_size The attribute is var sized or not.
-   * @param nullable The attribute is nullable or not.
-   * @param previous_result_bitmask The bitmask to get the previous result.
-   * @param current_result_bitmask The bitmask to set the current result.
    * @param result_buffer The result buffer.
    */
   template <typename T, QueryConditionOp Op>
@@ -418,9 +416,6 @@ class QueryCondition {
       const uint64_t src_cell,
       const uint64_t stride,
       const bool var_size,
-      const bool nullable,
-      const uint8_t previous_result_bitmask,
-      const uint8_t current_result_bitmask,
       uint8_t* result_buffer) const;
 
   /**
@@ -433,10 +428,8 @@ class QueryCondition {
    * @param src_cell The cell offset in the source tile.
    * @param stride The stride between cells.
    * @param var_size The attribute is var sized or not.
-   * @param nullable The attribute is nullable or not.
-   * @param previous_result_bitmask The bitmask to get the previous result.
-   * @param current_result_bitmask The bitmask to set the current result.
    * @param result_buffer The result buffer.
+   * @return Status.
    */
   template <typename T>
   Status apply_clause_dense(
@@ -447,9 +440,6 @@ class QueryCondition {
       const uint64_t src_cell,
       const uint64_t stride,
       const bool var_size,
-      const bool nullable,
-      const uint8_t previous_result_bitmask,
-      const uint8_t current_result_bitmask,
       uint8_t* result_buffer) const;
 
   /**
@@ -463,20 +453,17 @@ class QueryCondition {
    * @param length The number of cells to process.
    * @param src_cell The cell offset in the source tile.
    * @param stride The stride between cells.
-   * @param previous_result_bitmask The bitmask to get the previous result.
-   * @param current_result_bitmask The bitmask to set the current result.
    * @param result_buffer The result buffer.
+   * @return Status.
    */
   Status apply_clause_dense(
       const QueryCondition::Clause& clause,
-      const ArraySchema* const array_schema,
+      const ArraySchema& array_schema,
       ResultTile* result_tile,
       const uint64_t start,
       const uint64_t length,
       const uint64_t src_cell,
       const uint64_t stride,
-      const uint8_t previous_result_bitmask,
-      const uint8_t current_result_bitmask,
       uint8_t* result_buffer) const;
 
   /**
@@ -502,6 +489,7 @@ class QueryCondition {
    * @param result_tile The result tile to get the cells from.
    * @param var_size The attribute is var sized or not.
    * @param result_bitmap The result bitmap.
+   * @return Status.
    */
   template <typename T, typename BitmapType>
   Status apply_clause_sparse(
@@ -518,11 +506,12 @@ class QueryCondition {
    * @param array_schema The current array schema.
    * @param result_tile The result tile to get the cells from.
    * @param result_bitmap The result bitmap.
+   * @return Status.
    */
   template <typename BitmapType>
   Status apply_clause_sparse(
       const QueryCondition::Clause& clause,
-      const ArraySchema* const array_schema,
+      const ArraySchema& array_schema,
       ResultTile& result_tile,
       std::vector<BitmapType>& result_bitmap) const;
 };
