@@ -498,10 +498,10 @@ Status index_read_state_to_capnp(
       read_state->done_adding_result_tiles_);
 
   auto frag_tile_idx_builder =
-      read_state_builder.initFragTileIdx(read_state->frag_tile_idx_.size());
-  for (size_t i = 0; i < read_state->frag_tile_idx_.size(); ++i) {
-    frag_tile_idx_builder[i].setTileIdx(read_state->frag_tile_idx_[i].first);
-    frag_tile_idx_builder[i].setCellIdx(read_state->frag_tile_idx_[i].second);
+      read_state_builder.initFragTileIdx(read_state->frag_idx_.size());
+  for (size_t i = 0; i < read_state->frag_idx_.size(); ++i) {
+    frag_tile_idx_builder[i].setTileIdx(read_state->frag_idx_[i].tile_idx_);
+    frag_tile_idx_builder[i].setCellIdx(read_state->frag_idx_[i].cell_idx_);
   }
 
   return Status::Ok();
@@ -567,12 +567,12 @@ Status index_read_state_from_capnp(
       read_state_reader.getDoneAddingResultTiles();
 
   assert(read_state_reader.hasFragTileIdx());
-  read_state->frag_tile_idx_.clear();
+  read_state->frag_idx_.clear();
   for (const auto rcs : read_state_reader.getFragTileIdx()) {
     auto tile_idx = rcs.getTileIdx();
     auto cell_idx = rcs.getCellIdx();
 
-    read_state->frag_tile_idx_.emplace_back(tile_idx, cell_idx);
+    read_state->frag_idx_.emplace_back(tile_idx, cell_idx);
   }
 
   return Status::Ok();
