@@ -41,21 +41,23 @@
 #include <map>
 #include <mutex>
 #include <queue>
+#include <set>
 #include <string>
 #include <thread>
-#include <unordered_map>
 
+#include "tiledb/common/common.h"
 #include "tiledb/common/heap_memory.h"
 #include "tiledb/common/logger_public.h"
 #include "tiledb/common/status.h"
 #include "tiledb/common/thread_pool.h"
 #include "tiledb/sm/config/config.h"
 #include "tiledb/sm/enums/walk_order.h"
-#include "tiledb/sm/filesystem/filelock.h"
-#include "tiledb/sm/fragment/single_fragment_info.h"
+#include "tiledb/sm/filesystem/uri.h"
+#include "tiledb/sm/group/group.h"
 #include "tiledb/sm/misc/cancelable_tasks.h"
-#include "tiledb/sm/misc/uri.h"
+#include "tiledb/sm/misc/types.h"
 #include "tiledb/sm/stats/global_stats.h"
+#include "tiledb/sm/tile/filtered_buffer.h"
 
 using namespace tiledb::common;
 
@@ -71,12 +73,14 @@ class Consolidator;
 class EncryptionKey;
 class FragmentMetadata;
 class FragmentInfo;
+class Group;
 class Metadata;
 class OpenArray;
-class OpenArrayMemoryTracker;
+class MemoryTracker;
 class Query;
 class RestClient;
 class VFS;
+class ArrayDirectory;
 
 enum class EncryptionType : uint8_t;
 enum class ObjectType : uint8_t;
@@ -117,7 +121,7 @@ class StorageManager {
       ThreadPool* compute_tp,
       ThreadPool* io_tp,
       stats::Stats* parent_stats,
-      tdb_shared_ptr<Logger> logger);
+      shared_ptr<Logger> logger);
 
   /** Destructor. */
   ~StorageManager();
@@ -132,39 +136,88 @@ class StorageManager {
   /**
    * Closes an array opened for reads.
    *
-   * @param array_uri The array URI
+   * @param array The array to be closed.
    * @return Status
    */
-  Status array_close_for_reads(const URI& array_uri);
+  Status array_close_for_reads(Array* array);
 
   /**
    * Closes an array opened for writes.
    *
-   * @param array_uri The array URI
-   * @param encryption_key The array encryption key.
-   * @param array_metadata The array metadata, which the function
-   *     flush to persistent storage.
+   * @param array The array to be closed.
    * @return Status
    */
-  Status array_close_for_writes(
-      const URI& array_uri,
-      const EncryptionKey& encryption_key,
-      Metadata* array_metadata);
+  Status array_close_for_writes(Array* array);
 
   /**
-   * Opens an array for reads at a timestamp. All the metadata of the
-   * fragments created before or at the input timestamp are retrieved.
+   * Closes an group opened for reads.
    *
-   * If a timestamp_start is provided, this API will open the array between
-   * `timestamp_start` and `timestamp_end`.
+   * @param group The group to be closed.
+   * @return Status
+   */
+  Status group_close_for_reads(tiledb::sm::Group* group);
+
+  /**
+   * Closes an group opened for writes.
    *
-   * @param array_uri The array URI.
+   * @param group The group to be closed.
+   * @return Status
+   */
+  Status group_close_for_writes(tiledb::sm::Group* group);
+
+  /**
+   * Loads the group metadata from persistent storage based on
+   * the input URI manager.
+   */
+  Status load_group_metadata(
+      const tdb_shared_ptr<GroupDirectory>& group_dir,
+      const EncryptionKey& encryption_key,
+      Metadata* metadata);
+
+  /**
+   * Load a group detail from URI
+   *
+   * @param group_uri group uri
+   * @param uri location to load
+   * @param encryption_key encryption key
+   * @return tuple Status and pointer to group deserialized
+   */
+  tuple<Status, optional<shared_ptr<Group>>> load_group_from_uri(
+      const URI& group_uri,
+      const URI& uri,
+      const EncryptionKey& encryption_key);
+
+  /**
+   * Load group details based on group directory
+   *
+   * @param group_directory
+   * @param encryption_key encryption key
+   *
+   * @return tuple Status and pointer to group deserialized
+   */
+  tuple<Status, optional<shared_ptr<Group>>> load_group_details(
+      const shared_ptr<GroupDirectory>& group_directory,
+      const EncryptionKey& encryption_key);
+
+  /**
+   * Store the group details
+   *
+   * @param group to serialize and store
+   * @param encryption_key encryption key for at-rest encryption
+   * @return status
+   */
+  Status store_group_detail(Group* group, const EncryptionKey& encryption_key);
+
+  /**
+   * Returns the array schemas and fragment metadata for the given array.
+   * The function will focus only on relevant schemas and metadata as
+   * dictated by the input URI manager.
+   *
+   * @param array_dir The ArrayDirectory object used to retrieve the
+   *     various URIs in the array directory.
+   * @param memory_tracker The memory tracker of the array
+   *     for which the fragment metadata is loaded.
    * @param enc_key The encryption key to use.
-   * @param timestamp_start The (optional) starting timestamp to open the array
-   * between, starting at this timestamp and ending at `timestamp_end`.
-   * @param timestamp_end The timestamp at which the array will be opened.
-   *     In TileDB, timestamps are in ms elapsed since
-   *     1970-01-01 00:00:00 +0000 (UTC).
    * @return tuple of Status, latest ArraySchema, map of all array schemas and
    * vector of FragmentMetadata
    *        Status Ok on success, else error
@@ -174,82 +227,111 @@ class StorageManager {
    *        fragment_metadata The fragment metadata to be retrieved
    *           after the array is opened.
    */
-  std::tuple<
+  tuple<
       Status,
-      std::optional<ArraySchema*>,
-      std::optional<
-          std::unordered_map<std::string, tdb_shared_ptr<ArraySchema>>>,
-      std::optional<std::vector<tdb_shared_ptr<FragmentMetadata>>>>
-  array_open_for_reads(
-      const URI& array_uri,
-      const EncryptionKey& enc_key,
-      uint64_t timestamp_start,
-      uint64_t timestamp_end);
+      optional<shared_ptr<ArraySchema>>,
+      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>,
+      optional<std::vector<shared_ptr<FragmentMetadata>>>>
+  load_array_schemas_and_fragment_metadata(
+      const ArrayDirectory& array_dir,
+      MemoryTracker* memory_tracker,
+      const EncryptionKey& enc_key);
+
+  /**
+   * Opens an array for reads at a timestamp. All the metadata of the
+   * fragments created before or at the input timestamp are retrieved.
+   *
+   * If a timestamp_start is provided, this API will open the array between
+   * `timestamp_start` and `timestamp_end`.
+   *
+   * @param array The array to be opened.
+   * @return tuple of Status, latest ArraySchema, map of all array schemas and
+   * vector of FragmentMetadata
+   *        Status Ok on success, else error
+   *        ArraySchema The array schema to be retrieved after the
+   *           array is opened.
+   *        ArraySchemaMap Map of all array schemas found keyed by name
+   *        fragment_metadata The fragment metadata to be retrieved
+   *           after the array is opened.
+   */
+  tuple<
+      Status,
+      optional<shared_ptr<ArraySchema>>,
+      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>,
+      optional<std::vector<shared_ptr<FragmentMetadata>>>>
+  array_open_for_reads(Array* array);
 
   /**
    * Opens an array for reads without fragments.
    *
-   * @param array_uri The array URI.
-   * @param enc_key The encryption key to use.
+   * @param array The array to be opened.
    * @return tuple of Status, latest ArraySchema and map of all array schemas
    *        Status Ok on success, else error
    *        ArraySchema The array schema to be retrieved after the
    *          array is opened.
    *        ArraySchemaMap Map of all array schemas found keyed by name
    */
-  std::tuple<
+  tuple<
       Status,
-      std::optional<ArraySchema*>,
-      std::optional<
-          std::unordered_map<std::string, tdb_shared_ptr<ArraySchema>>>>
-  array_open_for_reads_without_fragments(
-      const URI& array_uri, const EncryptionKey& enc_key);
+      optional<shared_ptr<ArraySchema>>,
+      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>>
+  array_open_for_reads_without_fragments(Array* array);
 
   /** Opens an array for writes.
    *
-   * @param array_uri The array URI.
-   * @param enc_key The encryption key.
+   * @param array The array to open.
    * @return tuple of Status, latest ArraySchema and map of all array schemas
    *        Status Ok on success, else error
    *        ArraySchema The array schema to be retrieved after the
    *          array is opened.
    *        ArraySchemaMap Map of all array schemas found keyed by name
    */
+  tuple<
+      Status,
+      optional<shared_ptr<ArraySchema>>,
+      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>>
+  array_open_for_writes(Array* array);
+
+  /**
+   * Opens an group for reads.
+   *
+   * @param group The group to be opened.
+   * @return tuple of Status, latest GroupSchema and map of all group schemas
+   *        Status Ok on success, else error
+   */
   std::tuple<
       Status,
-      std::optional<ArraySchema*>,
       std::optional<
-          std::unordered_map<std::string, tdb_shared_ptr<ArraySchema>>>>
-  array_open_for_writes(const URI& array_uri, const EncryptionKey& enc_key);
+          const std::unordered_map<std::string, tdb_shared_ptr<GroupMember>>>>
+  group_open_for_reads(Group* group);
+
+  /** Opens an group for writes.
+   *
+   * @param group The group to open.
+   * @return tuple of Status, latest GroupSchema and map of all group schemas
+   *        Status Ok on success, else error
+   */
+  std::tuple<
+      Status,
+      std::optional<
+          const std::unordered_map<std::string, tdb_shared_ptr<GroupMember>>>>
+  group_open_for_writes(Group* group);
 
   /**
    * Load fragments for an already open array.
    *
-   * @param array_uri The array URI.
-   * @param enc_key The encryption key to use.
-   * @param fragment_metadata The fragment metadata to be retrieved
-   *          after the array is opened.
+   * @param array The open array.
    * @param fragment_info The list of fragment info.
-   * @return Status
+   * @return Status, the fragment metadata to be loaded.
    */
-  Status array_load_fragments(
-      const URI& array_uri,
-      const EncryptionKey& enc_key,
-      std::vector<tdb_shared_ptr<FragmentMetadata>>* fragment_metadata,
-      const std::vector<TimestampedURI>& fragment_info);
+  tuple<Status, optional<std::vector<shared_ptr<FragmentMetadata>>>>
+  array_load_fragments(
+      Array* array, const std::vector<TimestampedURI>& fragment_info);
 
   /**
-   * Reopens an already open array at a potentially new timestamp,
-   * retrieving the fragment metadata of any new fragments written
-   * in the array.
+   * Reopen an array for reads.
    *
-   * @param array_uri The array URI.
-   * @param enc_key The encryption key to use.
-   * @param timestamp_start The optional first timestamp between which the
-   *     array will be opened.
-   * @param timestamp_end The timestamp at which the array will be opened.
-   *     In TileDB, timestamps are in ms elapsed since
-   *     1970-01-01 00:00:00 +0000 (UTC).
+   * @param array The array to reopen.
    * @return tuple of Status, latest ArraySchema, map of all array schemas and
    * vector of FragmentMetadata
    *        Status Ok on success, else error
@@ -259,17 +341,12 @@ class StorageManager {
    *        FragmentMetadata The fragment metadata to be retrieved
    *          after the array is opened.
    */
-  std::tuple<
+  tuple<
       Status,
-      std::optional<ArraySchema*>,
-      std::optional<
-          std::unordered_map<std::string, tdb_shared_ptr<ArraySchema>>>,
-      std::optional<std::vector<tdb_shared_ptr<FragmentMetadata>>>>
-  array_reopen(
-      const URI& array_uri,
-      const EncryptionKey& enc_key,
-      uint64_t timestamp_start,
-      uint64_t timestamp_end);
+      optional<shared_ptr<ArraySchema>>,
+      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>,
+      optional<std::vector<shared_ptr<FragmentMetadata>>>>
+  array_reopen(Array* array);
 
   /**
    * Consolidates the fragments of an array into a single one.
@@ -291,6 +368,28 @@ class StorageManager {
       uint32_t key_length,
       const Config* config);
 
+  /**
+   * Consolidates the fragments of an array into a single one.
+   *
+   * @param array_name The name of the array to be consolidated.
+   * @param encryption_type The encryption type of the array
+   * @param encryption_key If the array is encrypted, the private encryption
+   *    key. For unencrypted arrays, pass `nullptr`.
+   * @param key_length The length in bytes of the encryption key.
+   * @param fragment_uris URIs of the fragments to consolidate.
+   * @param config Configuration parameters for the consolidation
+   *     (`nullptr` means default, which will use the config associated with
+   *      this instance).
+   * @return Status
+   */
+  Status fragments_consolidate(
+      const char* array_name,
+      EncryptionType encryption_type,
+      const void* encryption_key,
+      uint32_t key_length,
+      const std::vector<std::string> fragment_uris,
+      const Config* config);
+
   /**
    * Cleans up the array, such as its consolidated fragments and array
    * metadata. Note that this will coarsen the granularity of time traveling
@@ -302,38 +401,6 @@ class StorageManager {
    */
   Status array_vacuum(const char* array_name, const Config* config);
 
-  /**
-   * Cleans up fragments that took part in consolidation. Note that this
-   * will coarsen the granularity of time traveling (see docs for more
-   * information).
-   *
-   * @param array_name The name of the array to be vacuumed.
-   * @param timestamp_start The timestamp to start vacuuming at.
-   * @param timestamp_end The timestamp to end vacuuming at.
-   * @return Status
-   */
-  Status array_vacuum_fragments(
-      const char* array_name, uint64_t timestamp_start, uint64_t timestamp_end);
-
-  /**
-   * Cleans up consolidated fragment metadata (all except the last one).
-   *
-   * @param array_name The name of the array to be consolidated.
-   * @return Status
-   */
-  Status array_vacuum_fragment_meta(const char* array_name);
-
-  /**
-   * Cleans up consolidated array metadata.
-   *
-   * @param array_name The name of the array to be consolidated.
-   * @param timestamp_start The timestamp to start vacuuming at.
-   * @param timestamp_end The timestamp to end vacuuming at.
-   * @return Status
-   */
-  Status array_vacuum_array_meta(
-      const char* array_name, uint64_t timestamp_start, uint64_t timestamp_end);
-
   /**
    * Consolidates the metadata of an array into a single file.
    *
@@ -365,42 +432,35 @@ class StorageManager {
    */
   Status array_create(
       const URI& array_uri,
-      ArraySchema* array_schema,
+      const shared_ptr<ArraySchema>& array_schema,
       const EncryptionKey& encryption_key);
 
   /**
    * Evolve a TileDB array schema and store its new schema.
    *
-   * @param array_uri The URI of the array to be evolved.
+   * @param array_dir The ArrayDirectory object used to retrieve the
+   *     various URIs in the array directory.
    * @param schema_evolution The schema evolution.
    * @param encryption_key The encryption key to use.
    * @return Status
    */
   Status array_evolve_schema(
-      const URI& array_uri,
+      const ArrayDirectory& array_dir,
       ArraySchemaEvolution* array_schema,
       const EncryptionKey& encryption_key);
 
   /**
    * Upgrade a TileDB array to latest format version.
    *
-   * @param array_uri The URI of the array to be upgraded.
+   * @param array_dir The ArrayDirectory object used to retrieve the
+   *     various URIs in the array directory.
    * @param config Configuration parameters for the upgrade
    *     (`nullptr` means default, which will use the config associated with
    *      this instance).
    * @return Status
    */
-  Status array_upgrade_version(const URI& array_uri, const Config* config);
-
-  /**
-   * Gets the memory tracker for an open array.
-   *
-   * @param array_uri The array URI.
-   * @param top_level Specifies if the call is the top level call in recursion.
-   * @return The memory tracker.
-   */
-  OpenArrayMemoryTracker* array_memory_tracker(
-      const URI& array_uri, bool top_level = true);
+  Status array_upgrade_version(
+      const ArrayDirectory& array_dir, const Config* config);
 
   /**
    * Retrieves the non-empty domain from an array. This is the union of the
@@ -530,33 +590,13 @@ class StorageManager {
   /**
    * Retrieves the encryption type from an array.
    *
-   * @param array_uri URI of the array
+   * @param array_dir The ArrayDirectory object used to retrieve the
+   *     various URIs in the array directory.
    * @param encryption_type Set to the encryption type of the array.
    * @return Status
    */
   Status array_get_encryption(
-      const std::string& array_uri, EncryptionType* encryption_type);
-
-  /**
-   * Exclusively locks an array preventing it from being opened in
-   * read mode. This function will wait on the array to
-   * be closed if it is already open (always in read mode). After an array
-   * is xlocked, any attempt to open an array in read mode will have to wait
-   * until the array is unlocked with `xunlock_array`.
-   *
-   * An array is exclusively locked only for a short time upon consolidation,
-   * during removing the directories of the old fragments that got consolidated.
-   *
-   * @note Arrays that are opened in write mode need not be xlocked. The
-   *     reason is that the `OpenArray` objects created when opening
-   *     in write mode do not store any fragment metadata and, hence,
-   *     are not affected by a potentially concurrent consolidator deleting
-   *     fragment directories.
-   */
-  Status array_xlock(const URI& array_uri);
-
-  /** Releases an exclusive lock for the input array. */
-  Status array_xunlock(const URI& array_uri);
+      const ArrayDirectory& array_dir, EncryptionType* encryption_type);
 
   /**
    * Pushes an async query to the queue.
@@ -581,50 +621,6 @@ class StorageManager {
   /** Creates an empty file with the input URI. */
   Status touch(const URI& uri);
 
-  /**
-   * Gets the fragment information for a given array at a particular
-   * timestamp.
-   *
-   * @param array The array.
-   * @param timestamp_start The function will consider fragments created
-   *     at or after this timestamp.
-   * @param timestamp_end The function will consider fragments created
-   *     at or before this timestamp.
-   * @param fragment_info The fragment information to be retrieved.
-   *     The fragments are sorted in chronological creation order.
-   * @param get_to_vacuum Whether or not to receive information about
-   *     fragments to vacuum.
-   * @return Status
-   */
-  Status get_fragment_info(
-      const Array& array,
-      uint64_t timestamp_start,
-      uint64_t timestamp_end,
-      FragmentInfo* fragment_info,
-      bool get_to_vacuum = false);
-
-  /**
-   * Gets the fragment info for a single fragment URI.
-   *
-   * @param array The array.
-   * @param fragment_uri The fragment URI.
-   * @param fragment_info The fragment info to retrieve.
-   * @return Status
-   */
-  Status get_fragment_info(
-      const Array& array,
-      const URI& fragment_uri,
-      SingleFragmentInfo* fragment_info);
-
-  /**
-   * Retrieves all the fragment URIs of an array, along with the latest
-   * consolidated fragment metadata URI `meta_uri`.
-   */
-  Status get_fragment_uris(
-      const URI& array_uri,
-      std::vector<URI>* fragment_uris,
-      URI* meta_uri) const;
-
   /** Returns the current map of any set tags. */
   const std::unordered_map<std::string, std::string>& tags() const;
 
@@ -675,24 +671,6 @@ class StorageManager {
    */
   Status is_dir(const URI& uri, bool* is_dir) const;
 
-  /**
-   * Checks if the input URI represents a fragment. The functions takes into
-   * account the fragment version, which is retrived directly from the URI.
-   * For versions >= 5, the function checks whether the URI is included
-   * in `ok_uris`. For versions < 5, `ok_uris` is empty so the function
-   * checks for the existence of the fragment metadata file in the fragment
-   * URI directory. Therefore, the function is more expensive for earlier
-   * fragment versions.
-   *
-   * @param The URI to be checked.
-   * @param ok_uris For checking URI existence of versions >= 5.
-   * @param is_fragment Set to `1` if the URI is a fragment and `0`
-   *     otherwise.
-   * @return Status
-   */
-  Status is_fragment(
-      const URI& uri, const std::set<URI>& ok_uris, int* is_fragment) const;
-
   /**
    * Checks if the input URI represents a group.
    *
@@ -713,87 +691,70 @@ class StorageManager {
    */
   Status is_file(const URI& uri, bool* is_file) const;
 
-  /**
-   * Check if a URI is a vacuum file or not based on the file suffix
-   * @param uri
-   * @return true is vacuum file, false otherwise
-   */
-  bool is_vacuum_file(const URI& uri) const;
-
-  /**
-   * Retrieve all array schemas for an array uri under its __schema directory.
-   *
-   * @param array_uri The URI path of the array.
-   * @param uris The vector of array schema URIS sorted from earliest to the
-   * latest.
-   * @return Status
-   */
-  Status get_array_schema_uris(
-      const URI& array_uri, std::vector<URI>* schema_uris) const;
-
-  /**
-   * Get latest array schema for an array uri.
-   *
-   * @param array_uri The URI path of the array.
-   * @param uri The latest array schema URI.
-   * @return Status
-   */
-  Status get_latest_array_schema_uri(
-      const URI& array_uri, URI* schema_uri) const;
-
   /**
    * Loads the schema of a schema uri from persistent storage into memory.
    *
    * @param array_schema_uri The URI path of the array schema.
-   * @param array_uri The URI path of the array.
    * @param encryption_key The encryption key to use.
-   * @param array_schema The array schema to be retrieved.
-   * @return Status
+   * @return Status, the loaded array schema
    */
-  Status load_array_schema_from_uri(
-      const URI& array_schema_uri,
-      const EncryptionKey& encryption_key,
-      ArraySchema** array_schema);
+  tuple<Status, optional<shared_ptr<ArraySchema>>> load_array_schema_from_uri(
+      const URI& array_schema_uri, const EncryptionKey& encryption_key);
 
   /**
-   * Loads the schema of an array from persistent storage into memory.
+   * Loads the latest schema of an array from persistent storage into memory.
    *
-   * @param array_uri The URI path of the array.
+   * @param array_dir The ArrayDirectory object used to retrieve the
+   *     various URIs in the array directory.
    * @param encryption_key The encryption key to use.
-   * @param array_schema The array schema to be retrieved.
-   * @return Status
+   * @return Status, a new ArraySchema
    */
-  Status load_array_schema(
-      const URI& array_uri,
-      const EncryptionKey& encryption_key,
-      ArraySchema** array_schema);
+  tuple<Status, optional<shared_ptr<ArraySchema>>> load_array_schema_latest(
+      const ArrayDirectory& array_dir, const EncryptionKey& encryption_key);
+
+  /**
+   * It loads and returns the latest schema and all the array schemas
+   * (in the presence of schema evolution).
+   *
+   * @param array_dir The ArrayDirectory object used to retrieve the
+   *     various URIs in the array directory.
+   * @param encryption_key The encryption key to use.
+   * @return tuple of Status, latest array schema and all array schemas.
+   *   Status Ok on success, else error
+   *   ArraySchema The latest array schema.
+   *   ArraySchemaMap Map of all array schemas loaded, keyed by name
+   */
+  tuple<
+      Status,
+      optional<shared_ptr<ArraySchema>>,
+      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>>
+  load_array_schemas(
+      const ArrayDirectory& array_dir, const EncryptionKey& encryption_key);
 
   /**
    * Loads all schemas of an array from persistent storage into memory.
    *
-   * @param array_uri The URI path of the array.
+   * @param array_dir The ArrayDirectory object used to retrieve the
+   *     various URIs in the array directory.
    * @param encryption_key The encryption key to use.
    * @return tuple of Status and optional unordered map. If Status is an error
    * the unordered_map will be nullopt
    *        Status Ok on success, else error
    *        ArraySchemaMap Map of all array schemas found keyed by name
    */
-  std::tuple<
+  tuple<
       Status,
-      std::optional<
-          std::unordered_map<std::string, tdb_shared_ptr<ArraySchema>>>>
+      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>>
   load_all_array_schemas(
-      const URI& array_uri, const EncryptionKey& encryption_key);
+      const ArrayDirectory& array_dir, const EncryptionKey& encryption_key);
 
   /**
-   * Loads the array metadata from persistent storage that were created
-   * at or before `timestamp_end` and at or after `timestamp_start`.
+   * Loads the array metadata from persistent storage based on
+   * the input URI manager.
    */
   Status load_array_metadata(
-      const URI& array_uri,
+      const ArrayDirectory& array_dir,
       const EncryptionKey& encryption_key,
-      uint64_t timestamp_start,
-      uint64_t timestamp_end,
       Metadata* metadata);
 
   /** Removes a TileDB object (group, array). */
@@ -917,7 +878,7 @@ class StorageManager {
   Status read_from_cache(
       const URI& uri,
       uint64_t offset,
-      Buffer* buffer,
+      FilteredBuffer& buffer,
       uint64_t nbytes,
       bool* in_cache) const;
 
@@ -965,20 +926,19 @@ class StorageManager {
    * @return Status
    */
   Status store_array_schema(
-      ArraySchema* array_schema, const EncryptionKey& encryption_key);
+      const shared_ptr<ArraySchema>& array_schema,
+      const EncryptionKey& encryption_key);
 
   /**
-   * Stores the array metadata into persistent storage.
+   * Stores the metadata into persistent storage.
    *
-   * @param array_uri The array URI.
+   * @param uri The object URI.
    * @param encryption_key The encryption key to use.
-   * @param array_metadata The array metadata.
+   * @param metadata The  metadata.
    * @return Status
    */
-  Status store_array_metadata(
-      const URI& array_uri,
-      const EncryptionKey& encryption_key,
-      Metadata* array_metadata);
+  Status store_metadata(
+      const URI& uri, const EncryptionKey& encryption_key, Metadata* metadata);
 
   /** Closes a file, flushing its contents to persistent storage. */
   Status close_file(const URI& uri);
@@ -1001,7 +961,8 @@ class StorageManager {
    * @param buffer The buffer whose contents will be cached.
    * @return Status.
    */
-  Status write_to_cache(const URI& uri, uint64_t offset, Buffer* buffer) const;
+  Status write_to_cache(
+      const URI& uri, uint64_t offset, const FilteredBuffer& buffer) const;
 
   /**
    * Writes the contents of a buffer into a URI file.
@@ -1026,7 +987,7 @@ class StorageManager {
   stats::Stats* stats();
 
   /** Returns the internal logger object. */
-  tdb_shared_ptr<Logger> logger() const;
+  shared_ptr<Logger> logger() const;
 
  private:
   /* ********************************* */
@@ -1064,7 +1025,7 @@ class StorageManager {
   stats::Stats* stats_;
 
   /** The class logger. */
-  tdb_shared_ptr<Logger> logger_;
+  shared_ptr<Logger> logger_;
 
   /** Set to true when tasks are being cancelled. */
   bool cancellation_in_progress_;
@@ -1085,23 +1046,17 @@ class StorageManager {
   /** Stores the TileDB configuration parameters. */
   Config config_;
 
-  /** Stores exclusive filelocks for arrays. */
-  std::unordered_map<std::string, filelock_t> xfilelocks_;
+  /** Keeps track of which arrays are open. */
+  std::set<Array*> open_arrays_;
 
-  /** Mutex for managing OpenArray objects for reads. */
-  std::mutex open_array_for_reads_mtx_;
+  /** Mutex for managing open arrays. */
+  std::mutex open_arrays_mtx_;
 
-  /** Mutex for managing OpenArray objects for writes. */
-  std::mutex open_array_for_writes_mtx_;
+  /** Keeps track of which groups are open. */
+  std::set<Group*> open_groups_;
 
-  /** Mutex for managing exclusive locks. */
-  std::mutex xlock_mtx_;
-
-  /** Stores the currently open arrays for reads. */
-  std::map<std::string, OpenArray*> open_arrays_for_reads_;
-
-  /** Stores the currently open arrays for writes. */
-  std::map<std::string, OpenArray*> open_arrays_for_writes_;
+  /** Mutex for managing open groups. */
+  std::mutex open_groups_mtx_;
 
   /** Count of the number of queries currently in progress. */
   uint64_t queries_in_progress_;
@@ -1141,95 +1096,50 @@ class StorageManager {
   /*         PRIVATE METHODS           */
   /* ********************************* */
 
-  /**
-   * This is an auxiliary function to the other `array_open*` functions.
-   * It opens the array, retrieves an `OpenArray` instance, acquires
-   * its mutex and increases its counter. The array schema of the array
-   * is loaded, but not any fragment metadata at this point.
-   *
-   * @param array_uri The array URI.
-   * @param encryption_key The encryption key.
-   * @param open_array The `OpenArray` instance retrieved after opening
-   *      the array. Note that its mutex will be locked and its counter
-   *      will have been incremented when the function returns.
-   * @return Status
-   */
-  Status array_open_without_fragments(
-      const URI& array_uri,
-      const EncryptionKey& encryption_key,
-      OpenArray** open_array);
-
   /** Decrement the count of in-progress queries. */
   void decrement_in_progress();
 
-  /** Retrieves all the array metadata URI's of an array. */
-  Status get_array_metadata_uris(
-      const URI& array_uri, std::vector<URI>* array_metadata_uris) const;
-
   /** Increment the count of in-progress queries. */
   void increment_in_progress();
 
-  /**
-   * Loads the array schema into an open array.
-   *
-   * @param array_uri The array URI.
-   * @param open_array The open array object.
-   * @param encryption_key The encryption key to use.
-   * @return Status
-   */
-  Status load_array_schema(
-      const URI& array_uri,
-      OpenArray* open_array,
-      const EncryptionKey& encryption_key);
-
-  /**
-   * Loads the array metadata whose URIs are specified in the input.
-   * The array metadata are cached in binary form in `open_array`.
-   * Only the metadata that have not yet been loaded will be fetched
-   * into the open array object. Eventually, the function will
-   * deserialize the appropriate array metadata into `metadata`.
-   *
-   * @param open_array The open array object.
-   * @param encryption_key The encryption key to use.
-   * @param array_metadata_to_load The timestamped URIs of the array
-   *     metadata to load.
-   * @param metadata The array metadata to be retrieved.
-   * @return Status
-   */
-  Status load_array_metadata(
-      OpenArray* open_array,
-      const EncryptionKey& encryption_key,
-      const std::vector<TimestampedURI>& array_metadata_to_load,
-      Metadata* metadata);
-
   /**
    * Loads the fragment metadata of an open array given a vector of
-   * fragment URIs `fragments_to_load`. If the fragment metadata
-   * are not already loaded into the array, the function loads them.
+   * fragment URIs `fragments_to_load`.
    * The function stores the fragment metadata of each fragment
-   * in `fragments_to_load` into vector `fragment_metadata`, such
+   * in `fragments_to_load` into the returned vector, such
    * that there is a one-to-one correspondence between the two vectors.
    *
-   * @param open_array The open array object.
+   * If `meta_buf` has data, then some fragment metadata may be contained
+   * in there and does not need to be loaded from storage. In that
+   * case, `offsets` helps identifying each fragment metadata in the
+   * buffer.
+   *
+   * @param memory_tracker The memory tracker of the array
+   *     for which the metadata is loaded. This will be passed to
+   *     the constructor of each of the metadata loaded.
+   * @param array_schema_latest The latest array schema.
+   * @param array_schemas_all All the array schemas in a map keyed by the
+   *     schema filename.
    * @param encryption_key The encryption key to use.
    * @param fragments_to_load The fragments whose metadata to load.
-   * @param meta_buff A buffer that contains the consolidated fragment
-   *     metadata.
    * @param offsets A map from a fragment name to an offset in `meta_buff`
-   *     where the basic metadata can be found. If the offset cannot be
-   *     found, then the metadata of that fragment will be loaded from
+   *     where the basic fragment metadata can be found. If the offset
+   *     cannot be found, then the metadata of that fragment will be loaded from
    *     storage instead.
-   * @param fragment_metadata The fragment metadata retrieved in a
-   *     vector.
-   * @return Status
+   * @return tuple of Status and vector of FragmentMetadata
+   *        Status Ok on success, else error
+   *        Vector of FragmentMetadata is the fragment metadata to be retrieved.
    */
-  Status load_fragment_metadata(
-      OpenArray* open_array,
+  tuple<Status, optional<std::vector<shared_ptr<FragmentMetadata>>>>
+  load_fragment_metadata(
+      MemoryTracker* memory_tracker,
+      const shared_ptr<const ArraySchema>& array_schema,
+      const std::unordered_map<std::string, shared_ptr<ArraySchema>>&
+          array_schemas_all,
       const EncryptionKey& encryption_key,
       const std::vector<TimestampedURI>& fragments_to_load,
-      Buffer* meta_buff,
-      const std::unordered_map<std::string, uint64_t>& offsets,
-      std::vector<tdb_shared_ptr<FragmentMetadata>>* fragment_metadata);
+      const std::unordered_map<std::string, std::pair<Buffer*, uint64_t>>&
+          offsets);
 
   /**
    * Loads the latest consolidated fragment metadata from storage.
@@ -1237,52 +1147,12 @@ class StorageManager {
    * @param uri The URI of the consolidated fragment metadata.
    * @param enc_key The encryption key that may be needed to access the file.
    * @param f_buff The buffer to hold the consolidated fragment metadata.
-   * @param offsets A map from the fragment name to the offset in `f_buff` where
-   *     the basic fragment metadata starts.
-   * @return Status
+   * @return Status, vector from the fragment name to the offset in `f_buff`
+   *     where the basic fragment metadata starts.
    */
-  Status load_consolidated_fragment_meta(
-      const URI& uri,
-      const EncryptionKey& enc_key,
-      Buffer* f_buff,
-      std::unordered_map<std::string, uint64_t>* offsets);
-
-  /**
-   * Retrieves the URI of the latest consolidated fragment metadata,
-   * among the URIs in `uris`.
-   */
-  Status get_consolidated_fragment_meta_uri(
-      const std::vector<URI>& uris, URI* meta_uri) const;
-
-  /**
-   * Applicable to fragment and array metadata URIs.
-   *
-   * Gets the sorted URIs in ascending first timestamp order,
-   * breaking ties with lexicographic
-   * sorting of UUID. Only the URIs with timestamp between `timestamp_start`
-   * and `timestamp_end` (inclusive) are considered. The sorted URIs are
-   * stored in the last input, including their timestamps.
-   */
-  Status get_sorted_uris(
-      const std::vector<URI>& uris,
-      std::vector<TimestampedURI>* sorted_uris,
-      uint64_t timestamp_start,
-      uint64_t timestamp_end) const;
-
-  /**
-   * It computes the URIs `to_vacuum` from the input `uris`, considering
-   * only the URIs whose first timestamp is greater than or equal to
-   * `timestamp_start` and second timestamp is smaller than or equal to
-   * `timestamp_end`. The function also retrieves the `vac_uris` (files with
-   * `.vac` suffix) that were used to compute `to_vacuum`.
-   */
-  Status get_uris_to_vacuum(
-      const std::vector<URI>& uris,
-      uint64_t timestamp_start,
-      uint64_t timestamp_end,
-      std::vector<URI>* to_vacuum,
-      std::vector<URI>* vac_uris,
-      bool allow_partial = true) const;
+  tuple<Status, optional<std::vector<std::pair<std::string, uint64_t>>>>
+  load_consolidated_fragment_meta(
+      const URI& uri, const EncryptionKey& enc_key, Buffer* f_buff);
 
   /** Block until there are zero in-progress queries. */
   void wait_for_zero_in_progress();
