@@ -39,6 +39,7 @@
 #include "tiledb/sm/array_schema/domain.h"
 #include "tiledb/sm/enums/layout.h"
 #include "tiledb/sm/misc/hilbert.h"
+#include "tiledb/sm/misc/tdb_math.h"
 #include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/stats/global_stats.h"
 
@@ -71,7 +72,7 @@ SubarrayPartitioner::SubarrayPartitioner(
     const uint64_t memory_budget_validity,
     ThreadPool* const compute_tp,
     Stats* const parent_stats,
-    tdb_shared_ptr<Logger> logger)
+    shared_ptr<Logger> logger)
     : stats_(parent_stats->create_child("SubarrayPartitioner"))
     , logger_(logger->clone("SubarrayPartitioner", ++logger_id_))
     , config_(config)
@@ -150,41 +151,41 @@ Status SubarrayPartitioner::get_result_budget(
     const char* name, uint64_t* budget) const {
   // Check attribute/dimension name
   if (name == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget; Attribute/Dimension name cannot be null"));
 
   // Check budget pointer
   if (budget == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget; Invalid budget input"));
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  bool is_dim = array_schema->is_dim(name);
-  bool is_attr = array_schema->is_attr(name);
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  bool is_dim = array_schema.is_dim(name);
+  bool is_attr = array_schema.is_attr(name);
 
   // Check if attribute/dimension exists
   if (name != constants::coords && !is_dim && !is_attr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Invalid attribute/dimension '") +
         name + "'"));
 
   // Check if the attribute/dimension is fixed-sized
-  if (array_schema->var_size(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (array_schema.var_size(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Input attribute/dimension '") +
         name + "' is var-sized"));
 
   // Check if the attribute is nullable
-  if (array_schema->is_nullable(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (array_schema.is_nullable(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Input attribute/dimension '") +
         name + "' is nullable"));
 
   // Check if budget has been set
   auto b_it = budget_.find(name);
   if (b_it == budget_.end())
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Budget not set for "
                     "attribute/dimension '") +
         name + "'"));
@@ -199,47 +200,47 @@ Status SubarrayPartitioner::get_result_budget(
     const char* name, uint64_t* budget_off, uint64_t* budget_val) const {
   // Check attribute/dimension name
   if (name == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget; Attribute/Dimension name cannot be null"));
 
   // Check budget pointers
   if (budget_off == nullptr || budget_val == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget; Invalid budget input"));
 
   // Check zipped coordinates
   if (name == constants::coords)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget for zipped coordinates; Attribute/Dimension "
         "must be var-sized"));
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  bool is_dim = array_schema->is_dim(name);
-  bool is_attr = array_schema->is_attr(name);
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  bool is_dim = array_schema.is_dim(name);
+  bool is_attr = array_schema.is_attr(name);
 
   // Check if attribute/dimension exists
   if (!is_dim && !is_attr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Invalid attribute/dimension '") +
         name + "'"));
 
   // Check if the attribute/dimension is var-sized
-  if (!array_schema->var_size(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (!array_schema.var_size(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Input attribute/dimension '") +
         name + "' is fixed-sized"));
 
   // Check if the attribute is nullable
-  if (array_schema->is_nullable(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (array_schema.is_nullable(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Input attribute/dimension '") +
         name + "' is nullable"));
 
   // Check if budget has been set
   auto b_it = budget_.find(name);
   if (b_it == budget_.end())
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Budget not set for "
                     "attribute/dimension '") +
         name + "'"));
@@ -255,40 +256,40 @@ Status SubarrayPartitioner::get_result_budget_nullable(
     const char* name, uint64_t* budget, uint64_t* budget_validity) const {
   // Check attribute name
   if (name == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget; Attribute name cannot be null"));
 
   // Check budget pointers
   if (budget == nullptr || budget_validity == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget; Invalid budget input"));
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  bool is_attr = array_schema->is_attr(name);
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  bool is_attr = array_schema.is_attr(name);
 
   // Check if attribute exists
   if (!is_attr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Invalid attribute '") + name +
         "'"));
 
   // Check if the attribute is fixed-sized
-  if (array_schema->var_size(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (array_schema.var_size(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Input attribute '") + name +
         "' is var-sized"));
 
   // Check if the attribute is nullable
-  if (!array_schema->is_nullable(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (!array_schema.is_nullable(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Input attribute '") + name +
         "' is not nullable"));
 
   // Check if budget has been set
   auto b_it = budget_.find(name);
   if (b_it == budget_.end())
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Budget not set for "
                     "attribute '") +
         name + "'"));
@@ -307,41 +308,41 @@ Status SubarrayPartitioner::get_result_budget_nullable(
     uint64_t* budget_validity) const {
   // Check attribute/dimension name
   if (name == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget; Attribute/Dimension name cannot be null"));
 
   // Check budget pointers
   if (budget_off == nullptr || budget_val == nullptr ||
       budget_validity == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot get result budget; Invalid budget input"));
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  bool is_attr = array_schema->is_attr(name);
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  bool is_attr = array_schema.is_attr(name);
 
   // Check if attribute exists
   if (!is_attr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Invalid attribute '") + name +
         "'"));
 
   // Check if the attribute is var-sized
-  if (!array_schema->var_size(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (!array_schema.var_size(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Input attribute '") + name +
         "' is fixed-sized"));
 
   // Check if the attribute is nullable
-  if (!array_schema->is_nullable(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (!array_schema.is_nullable(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Input attribute '") + name +
         "' is not nullable"));
 
   // Check if budget has been set
   auto b_it = budget_.find(name);
   if (b_it == budget_.end())
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot get result budget; Budget not set for "
                     "attribute '") +
         name + "'"));
@@ -415,31 +416,31 @@ Status SubarrayPartitioner::set_result_budget(
     const char* name, uint64_t budget) {
   // Check attribute/dimension name
   if (name == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot set result budget; Attribute/Dimension name cannot be null"));
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  bool is_dim = array_schema->is_dim(name);
-  bool is_attr = array_schema->is_attr(name);
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  bool is_dim = array_schema.is_dim(name);
+  bool is_attr = array_schema.is_attr(name);
 
   // Check if attribute/dimension exists
   if (name != constants::coords && !is_dim && !is_attr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Invalid attribute/dimension '") +
         name + "'"));
 
   // Check if the attribute/dimension is fixed-sized
-  bool var_size = (name != constants::coords && array_schema->var_size(name));
+  bool var_size = (name != constants::coords && array_schema.var_size(name));
   if (var_size)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Input attribute/dimension '") +
         name + "' is var-sized"));
 
   // Check if the attribute/dimension is nullable
-  bool nullable = array_schema->is_nullable(name);
+  bool nullable = array_schema.is_nullable(name);
   if (nullable)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Input attribute/dimension '") +
         name + "' is nullable"));
 
@@ -452,35 +453,35 @@ Status SubarrayPartitioner::set_result_budget(
     const char* name, uint64_t budget_off, uint64_t budget_val) {
   // Check attribute/dimension name
   if (name == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot set result budget; Attribute/Dimension name cannot be null"));
 
   if (name == constants::coords)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot set result budget for zipped coordinates; Attribute/Dimension "
         "must be var-sized"));
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  bool is_dim = array_schema->is_dim(name);
-  bool is_attr = array_schema->is_attr(name);
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  bool is_dim = array_schema.is_dim(name);
+  bool is_attr = array_schema.is_attr(name);
 
   // Check if attribute/dimension exists
   if (!is_dim && !is_attr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Invalid attribute/dimension '") +
         name + "'"));
 
   // Check if the attribute/dimension is var-sized
-  if (!array_schema->var_size(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (!array_schema.var_size(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Input attribute/dimension '") +
         name + "' is fixed-sized"));
 
   // Check if the attribute/dimension is nullable
-  bool nullable = array_schema->is_nullable(name);
+  bool nullable = array_schema.is_nullable(name);
   if (nullable)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Input attribute/dimension '") +
         name + "' is nullable"));
 
@@ -493,30 +494,30 @@ Status SubarrayPartitioner::set_result_budget_nullable(
     const char* name, uint64_t budget, uint64_t budget_validity) {
   // Check attribute/dimension name
   if (name == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot set result budget; Attribute name cannot be null"));
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  bool is_attr = array_schema->is_attr(name);
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  bool is_attr = array_schema.is_attr(name);
 
   // Check if attribute exists
   if (!is_attr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Invalid attribute '") + name +
         "'"));
 
   // Check if the attribute is fixed-sized
-  bool var_size = array_schema->var_size(name);
+  bool var_size = array_schema.var_size(name);
   if (var_size)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Input attribute '") + name +
         "' is var-sized"));
 
   // Check if the attribute is nullable
-  bool nullable = array_schema->is_nullable(name);
+  bool nullable = array_schema.is_nullable(name);
   if (!nullable)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Input attribute '") + name +
         "' is not nullable"));
 
@@ -532,29 +533,29 @@ Status SubarrayPartitioner::set_result_budget_nullable(
     uint64_t budget_validity) {
   // Check attribute/dimension name
   if (name == nullptr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         "Cannot set result budget; Attribute name cannot be null"));
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  bool is_attr = array_schema->is_attr(name);
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  bool is_attr = array_schema.is_attr(name);
 
   // Check if attribute exists
   if (!is_attr)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Invalid attribute '") + name +
         "'"));
 
   // Check if the attribute is var-sized
-  if (!array_schema->var_size(name))
-    return logger_->status(Status::SubarrayPartitionerError(
+  if (!array_schema.var_size(name))
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Input attribute '") + name +
         "' is fixed-sized"));
 
   // Check if the attribute is nullable
-  bool nullable = array_schema->is_nullable(name);
+  bool nullable = array_schema.is_nullable(name);
   if (!nullable)
-    return logger_->status(Status::SubarrayPartitionerError(
+    return logger_->status(Status_SubarrayPartitionerError(
         std::string("Cannot set result budget; Input attribute '") + name +
         "' is not nullable"));
 
@@ -634,12 +635,12 @@ SubarrayPartitioner::State* SubarrayPartitioner::state() {
   return &state_;
 }
 
-const Subarray* SubarrayPartitioner::subarray() const {
-  return &subarray_;
+const Subarray& SubarrayPartitioner::subarray() const {
+  return subarray_;
 }
 
-Subarray* SubarrayPartitioner::subarray() {
-  return &subarray_;
+Subarray& SubarrayPartitioner::subarray() {
+  return subarray_;
 }
 
 stats::Stats* SubarrayPartitioner::stats() const {
@@ -672,7 +673,7 @@ Status SubarrayPartitioner::calibrate_current_start_end(bool* must_split_slab) {
   }
 
   auto layout = subarray_.layout();
-  auto cell_order = subarray_.array()->array_schema_latest()->cell_order();
+  auto cell_order = subarray_.array()->array_schema_latest().cell_order();
   cell_order = (cell_order == Layout::HILBERT) ? Layout::ROW_MAJOR : cell_order;
   layout = (layout == Layout::UNORDERED) ? cell_order : layout;
   assert(layout == Layout::ROW_MAJOR || layout == Layout::COL_MAJOR);
@@ -880,13 +881,13 @@ void SubarrayPartitioner::compute_splitting_value_on_tiles(
   *unsplittable = true;
 
   // Inapplicable to Hilbert cell order
-  if (subarray_.array()->array_schema_latest()->cell_order() == Layout::HILBERT)
+  if (subarray_.array()->array_schema_latest().cell_order() == Layout::HILBERT)
     return;
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  auto dim_num = subarray_.array()->array_schema_latest()->dim_num();
-  auto layout = subarray_.array()->array_schema_latest()->tile_order();
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  auto dim_num = array_schema.dim_num();
+  auto layout = array_schema.tile_order();
   *splitting_dim = UINT32_MAX;
 
   std::vector<unsigned> dims;
@@ -901,7 +902,7 @@ void SubarrayPartitioner::compute_splitting_value_on_tiles(
   // Compute splitting dimension and value
   const Range* r;
   for (auto d : dims) {
-    auto dim = array_schema->domain()->dimension(d);
+    auto dim = array_schema.domain().dimension(d);
     range.get_range(d, 0, &r);
     auto tiles_apart = dim->tile_num(*r) - 1;
     if (tiles_apart != 0) {
@@ -938,9 +939,9 @@ void SubarrayPartitioner::compute_splitting_value_single_range(
   }
 
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  auto dim_num = array_schema->dim_num();
-  auto cell_order = array_schema->cell_order();
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  auto dim_num = array_schema.dim_num();
+  auto cell_order = array_schema.cell_order();
   assert(!range.is_unary());
   auto layout = subarray_.layout();
   if (layout == Layout::UNORDERED && cell_order == Layout::HILBERT) {
@@ -974,7 +975,7 @@ void SubarrayPartitioner::compute_splitting_value_single_range(
   // Compute splitting dimension and value
   const Range* r;
   for (auto d : dims) {
-    auto dim = array_schema->dimension(d);
+    auto dim = array_schema.dimension(d);
     range.get_range(d, 0, &r);
     if (!r->unary()) {
       *splitting_dim = d;
@@ -998,8 +999,8 @@ void SubarrayPartitioner::compute_splitting_value_single_range_hilbert(
     bool* normal_order,
     bool* unsplittable) {
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  auto dim_num = array_schema->dim_num();
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  auto dim_num = array_schema.dim_num();
   Hilbert h(dim_num);
 
   // Compute the uint64 mapping of the range (bits properly shifted)
@@ -1018,7 +1019,7 @@ void SubarrayPartitioner::compute_splitting_value_single_range_hilbert(
       range_uint64[*splitting_dim], *splitting_dim, splitting_value);
 
   // Check for unsplittable again
-  auto dim = array_schema->dimension(*splitting_dim);
+  auto dim = array_schema.dimension(*splitting_dim);
   const Range* r;
   range.get_range(*splitting_dim, 0, &r);
   if (dim->smaller_than(*splitting_value, *r)) {
@@ -1059,11 +1060,11 @@ Status SubarrayPartitioner::compute_splitting_value_multi_range(
 
   // Multi-range partition
   auto layout = subarray_.layout();
-  auto array_schema = subarray_.array()->array_schema_latest();
-  auto dim_num = array_schema->dim_num();
-  auto cell_order = (array_schema->cell_order() == Layout::HILBERT) ?
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  auto dim_num = array_schema.dim_num();
+  auto cell_order = (array_schema.cell_order() == Layout::HILBERT) ?
                         Layout::ROW_MAJOR :
-                        array_schema->cell_order();
+                        array_schema.cell_order();
   layout = (layout == Layout::UNORDERED) ? cell_order : layout;
   *splitting_dim = UINT32_MAX;
   uint64_t range_num;
@@ -1092,7 +1093,7 @@ Status SubarrayPartitioner::compute_splitting_value_multi_range(
 
     // Check if we need to split single range
     partition.get_range(d, 0, &r);
-    auto dim = array_schema->dimension(d);
+    auto dim = array_schema.dimension(d);
     if (!r->unary()) {
       *splitting_dim = d;
       dim->splitting_value(*r, splitting_value, unsplittable);
@@ -1105,7 +1106,7 @@ Status SubarrayPartitioner::compute_splitting_value_multi_range(
 }
 
 bool SubarrayPartitioner::must_split(Subarray* partition) {
-  auto array_schema = subarray_.array()->array_schema_latest();
+  const auto& array_schema = subarray_.array()->array_schema_latest();
   bool must_split = false;
 
   uint64_t size_fixed;
@@ -1117,8 +1118,8 @@ bool SubarrayPartitioner::must_split(Subarray* partition) {
   for (const auto& b : budget_) {
     // Compute max sizes
     auto name = b.first;
-    auto var_size = array_schema->var_size(name);
-    auto nullable = array_schema->is_nullable(name);
+    auto var_size = array_schema.var_size(name);
+    auto nullable = array_schema.is_nullable(name);
 
     // Compute est sizes
     size_fixed = 0;
@@ -1150,6 +1151,8 @@ bool SubarrayPartitioner::must_split(Subarray* partition) {
         partition->get_max_memory_size(
             b.first.c_str(), &mem_size_fixed, config_, compute_tp_);
       } else {
+        partition->get_est_result_size_nullable(
+            b.first.c_str(), &size_fixed, &size_validity, config_, compute_tp_);
         partition->get_max_memory_size_nullable(
             b.first.c_str(),
             &mem_size_fixed,
@@ -1176,7 +1179,7 @@ bool SubarrayPartitioner::must_split(Subarray* partition) {
         }
       } else {
         if (!nullable) {
-          partition->get_est_result_size(
+          partition->get_est_result_size_internal(
               b.first.c_str(), &size_fixed, config_, compute_tp_);
         } else {
           partition->get_est_result_size_nullable(
@@ -1362,8 +1365,8 @@ void SubarrayPartitioner::compute_range_uint64(
     std::vector<std::array<uint64_t, 2>>* range_uint64,
     bool* unsplittable) const {
   // Initializations
-  auto array_schema = subarray_.array()->array_schema_latest();
-  auto dim_num = array_schema->dim_num();
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  auto dim_num = array_schema.dim_num();
   const Range* r;
   *unsplittable = true;
   range_uint64->resize(dim_num);
@@ -1377,7 +1380,7 @@ void SubarrayPartitioner::compute_range_uint64(
   // Calculate mapped range
   bool empty_start, empty_end;
   for (uint32_t d = 0; d < dim_num; ++d) {
-    auto dim = array_schema->dimension(d);
+    auto dim = array_schema.dimension(d);
     auto var = dim->var_size();
     range.get_range(d, 0, &r);
     empty_start = var ? (r->start_size() == 0) : r->empty();
@@ -1407,8 +1410,8 @@ void SubarrayPartitioner::compute_splitting_dim_hilbert(
     const std::vector<std::array<uint64_t, 2>>& range_uint64,
     uint32_t* splitting_dim) const {
   // For easy reference
-  auto array_schema = subarray_.array()->array_schema_latest();
-  auto dim_num = array_schema->dim_num();
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  auto dim_num = array_schema.dim_num();
 
   // Prepare candidate splitting dimensions
   std::set<uint32_t> splitting_dims;
@@ -1481,8 +1484,8 @@ void SubarrayPartitioner::compute_splitting_value_hilbert(
     const std::array<uint64_t, 2>& range_uint64,
     uint32_t splitting_dim,
     ByteVecValue* splitting_value) const {
-  auto array_schema = subarray_.array()->array_schema_latest();
-  auto dim_num = array_schema->dim_num();
+  const auto& array_schema = subarray_.array()->array_schema_latest();
+  auto dim_num = array_schema.dim_num();
   uint64_t splitting_value_uint64 = range_uint64[0];  // Splitting value
   if (range_uint64[0] + 1 != range_uint64[1]) {
     uint64_t left_p2_m1, right_p2_m1;  // Left/right powers of 2 minus 1
@@ -1522,6 +1525,6 @@ void SubarrayPartitioner::compute_splitting_value_hilbert(
   auto max_bucket_val = ((uint64_t)1 << bits) - 1;
 
   *splitting_value =
-      array_schema->dimension(splitting_dim)
+      array_schema.dimension(splitting_dim)
           ->map_from_uint64(splitting_value_uint64, bits, max_bucket_val);
 }
